@@ -175,7 +175,7 @@ void set_ldids(HINF hInf, LPCWSTR pszInstallSection, LPCWSTR pszWorkingDir)
             FIXME("Need to support changing paths - default will be used\n");
 
         /* set all ldids to dest */
-        while ((ptr = get_parameter(&key, ',')))
+        while ((ptr = get_parameter(&key, ',', FALSE)))
         {
             ldid = atolW(ptr);
             SetupSetDirectoryIdW(hInf, ldid, dest);
@@ -207,19 +207,6 @@ HRESULT WINAPI CloseINFEngine(HINF hInf)
     return S_OK;
 }
 
-/***********************************************************************
- *           DllMain (ADVPACK.@)
- */
-BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
-{
-    TRACE("(%p, %d, %p)\n", hinstDLL, fdwReason, lpvReserved);
-
-    if (fdwReason == DLL_PROCESS_ATTACH)
-        DisableThreadLibraryCalls(hinstDLL);
-
-    return TRUE;
-}
-
 /***********************************************************************
  *              IsNTAdmin	(ADVPACK.@)
  *
@@ -508,12 +495,12 @@ HRESULT WINAPI RegisterOCX(HWND hWnd, HINSTANCE hInst, LPCSTR cmdline, INT show)
     cmdline_ptr = cmdline_copy;
     lstrcpyW(cmdline_copy, cmdlineW.Buffer);
 
-    ocx_filename = get_parameter(&cmdline_ptr, ',');
+    ocx_filename = get_parameter(&cmdline_ptr, ',', TRUE);
     if (!ocx_filename || !*ocx_filename)
         goto done;
 
-    str_flags = get_parameter(&cmdline_ptr, ',');
-    param = get_parameter(&cmdline_ptr, ',');
+    str_flags = get_parameter(&cmdline_ptr, ',', TRUE);
+    param = get_parameter(&cmdline_ptr, ',', TRUE);
 
     hm = LoadLibraryExW(ocx_filename, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
     if (!hm)
@@ -692,7 +679,7 @@ HRESULT WINAPI TranslateInfStringA(LPCSTR pszInfFilename, LPCSTR pszInstallSecti
                                     dwBufferSize, NULL, NULL);
             }
             else
-                res = HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER);
+                res = E_NOT_SUFFICIENT_BUFFER;
         }
         
         HeapFree(GetProcessHeap(), 0, bufferW);
@@ -752,7 +739,7 @@ HRESULT WINAPI TranslateInfStringW(LPCWSTR pszInfFilename, LPCWSTR pszInstallSec
                            pszBuffer, dwBufferSize, pdwRequiredSize))
     {
         if (dwBufferSize < *pdwRequiredSize)
-            hret = HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER);
+            hret = E_NOT_SUFFICIENT_BUFFER;
         else
             hret = SPAPI_E_LINE_NOT_FOUND;
     }
@@ -809,7 +796,7 @@ HRESULT WINAPI TranslateInfStringExA(HINF hInf, LPCSTR pszInfFilename,
                                     dwBufferSize, NULL, NULL);
             }
             else
-                res = HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER);
+                res = E_NOT_SUFFICIENT_BUFFER;
         }
         
         HeapFree(GetProcessHeap(), 0, bufferW);
@@ -867,7 +854,7 @@ HRESULT WINAPI TranslateInfStringExW(HINF hInf, LPCWSTR pszInfFilename,
                            pszBuffer, dwBufferSize, pdwRequiredSize))
     {
         if (dwBufferSize < *pdwRequiredSize)
-            return HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER);
+            return E_NOT_SUFFICIENT_BUFFER;
 
         return SPAPI_E_LINE_NOT_FOUND;
     }
