@@ -1069,7 +1069,7 @@ HRESULT load_mesh_data(IDirect3DRMMeshBuilder3 *iface, IDirectXFileData *pData,
             return hr;
     }
 
-    TRACE("Mesh name is '%s'\n", This->name ? This->name : "");
+    TRACE("Mesh name is %s\n", debugstr_a(This->name));
 
     This->nb_normals = 0;
 
@@ -1162,6 +1162,7 @@ HRESULT load_mesh_data(IDirect3DRMMeshBuilder3 *iface, IDirectXFileData *pData,
             IDirectXFileObject *child;
             DWORD i = 0;
             float* values;
+            struct d3drm_texture *texture_object;
 
             TRACE("Process MeshMaterialList\n");
 
@@ -1234,7 +1235,8 @@ HRESULT load_mesh_data(IDirect3DRMMeshBuilder3 *iface, IDirectXFileData *pData,
 
                 values = (float*)ptr;
 
-                d3drm_set_color(&This->materials[i].color, values[0], values[1], values[2], values[3]);
+                This->materials[i].color = RGBA_MAKE((BYTE)(values[0] * 255.0f), (BYTE)(values[1] * 255.0f),
+                        (BYTE)(values[2] * 255.0f), (BYTE)(values[3] * 255.0f));
 
                 IDirect3DRMMaterial2_SetAmbient(This->materials[i].material, values[0], values [1], values[2]); /* Alpha ignored */
                 IDirect3DRMMaterial2_SetPower(This->materials[i].material, values[4]);
@@ -1298,13 +1300,12 @@ HRESULT load_mesh_data(IDirect3DRMMeshBuilder3 *iface, IDirectXFileData *pData,
                             if (file != INVALID_HANDLE_VALUE)
                             {
                                 CloseHandle(file);
-
-                                hr = Direct3DRMTexture_create(&IID_IDirect3DRMTexture3, (IUnknown**)&This->materials[i].texture);
-                                if (FAILED(hr))
+                                if (FAILED(hr = d3drm_texture_create(&texture_object)))
                                 {
                                     IDirectXFileData_Release(data);
                                     goto end;
                                 }
+                                This->materials[i].texture = &texture_object->IDirect3DRMTexture3_iface;
                             }
                         }
                     }
@@ -1693,7 +1694,7 @@ static HRESULT WINAPI d3drm_mesh_builder3_SetColorRGB(IDirect3DRMMeshBuilder3 *i
 
     TRACE("iface %p, red %.8e, green %.8e, blue %.8e.\n", iface, red, green, blue);
 
-    d3drm_set_color(&mesh_builder->color, red, green, blue, 1.0f);
+    mesh_builder->color = RGBA_MAKE((BYTE)(red * 255.0f), (BYTE)(green * 255.0f), (BYTE)(blue * 255.0f), 0xff);
 
     return D3DRM_OK;
 }
@@ -2594,7 +2595,7 @@ static HRESULT WINAPI d3drm_mesh_SetGroupColorRGB(IDirect3DRMMesh *iface,
     if (id >= mesh->nb_groups)
         return D3DRMERR_BADVALUE;
 
-    d3drm_set_color(&mesh->groups[id].color, red, green, blue, 1.0f);
+    mesh->groups[id].color = RGBA_MAKE((BYTE)(red * 255.0f), (BYTE)(green * 255.0f), (BYTE)(blue * 255.0f), 0xff);
 
     return D3DRM_OK;
 }
