@@ -19,6 +19,7 @@
  */
 
 #include "config.h"
+#include "wine/port.h"
 #include "wine/debug.h"
 
 #define COBJMACROS
@@ -90,7 +91,7 @@ static const char *debugstr_fourcc(DWORD fourcc)
         (char)(fourcc >> 16), (char)(fourcc >> 24));
 }
 
-static const char* get_primitive_string(WORD token)
+static const char* get_primitive_string(DWORD token)
 {
   switch(token)
   {
@@ -542,7 +543,7 @@ static BOOL is_float(parse_buffer* buf)
   DWORD pos = 0;
   char c;
   float decimal;
-  BOOL dot = 0;
+  BOOL dot = FALSE;
 
   while (pos < buf->rem_bytes && !is_separator(c = *(buf->buffer+pos)))
   {
@@ -899,10 +900,10 @@ static inline BOOL is_primitive_type(WORD token)
     case TOKEN_LPSTR:
     case TOKEN_UNICODE:
     case TOKEN_CSTRING:
-      ret = 1;
+      ret = TRUE;
       break;
     default:
-      ret = 0;
+      ret = FALSE;
       break;
   }
   return ret;
@@ -948,14 +949,14 @@ static BOOL parse_template_members_list(parse_buffer * buf)
 
   while (1)
   {
-    BOOL array = 0;
+    BOOL array = FALSE;
     int nb_dims = 0;
     cur_member = &buf->pdxf->xtemplates[buf->pdxf->nb_xtemplates].members[idx_member];
 
     if (check_TOKEN(buf) == TOKEN_ARRAY)
     {
       get_TOKEN(buf);
-      array = 1;
+      array = TRUE;
     }
 
     if (check_TOKEN(buf) == TOKEN_NAME)
@@ -1348,7 +1349,8 @@ _exit:
             return FALSE;
         }
 
-        buf->pxo = buf->pxo->children[buf->pxo->nb_children] = &buf->pxo_tab[buf->pxo->root->nb_subobjects++];
+        buf->pxo = buf->pxo->children[buf->pxo->nb_children] = &buf->pxo_tab[buf->pxo->root->nb_subobjects];
+        pxo->root->nb_subobjects++;
 
         TRACE("Enter optional %s\n", (char*)buf->value);
         buf->level++;
