@@ -157,13 +157,6 @@ void putieee32(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, floa
     *fbuf = value;
 }
 
-void putieee32_sum(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value)
-{
-    BYTE *buf = (BYTE *)dsb->device->tmp_buffer;
-    float *fbuf = (float*)(buf + pos + sizeof(float) * channel);
-    *fbuf += value;
-}
-
 void put_mono2stereo(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value)
 {
     dsb->put_aux(dsb, pos, 0, value);
@@ -213,45 +206,6 @@ void put_stereo2surround51(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD c
     }
 }
 
-void put_surround512stereo(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value)
-{
-    /* based on pulseaudio's downmix algorithm */
-    switch(channel){
-
-    case 4: /* back left */
-        value *= 0.056f; /* (1/9) / (sum of left volumes) */
-        dsb->put_aux(dsb, pos, 0, value);
-        break;
-
-    case 0: /* front left */
-        value *= 0.503; /* 1 / (sum of left volumes) */
-        dsb->put_aux(dsb, pos, 0, value);
-        break;
-
-    case 5: /* back right */
-        value *= 0.056f; /* (1/9) / (sum of right volumes) */
-        dsb->put_aux(dsb, pos, 1, value);
-        break;
-
-    case 1: /* front right */
-        value *= 0.503f; /* 1 / (sum of right volumes) */
-        dsb->put_aux(dsb, pos, 1, value);
-        break;
-
-    case 2: /* front centre */
-        value *= 0.252f; /* 0.5 / (sum of left/right volumes) */
-        dsb->put_aux(dsb, pos, 0, value);
-        dsb->put_aux(dsb, pos, 1, value);
-        break;
-
-    case 3: /* LFE */
-        value *= 0.189f; /* 0.375 / (sum of left/right volumes) */
-        dsb->put_aux(dsb, pos, 0, value);
-        dsb->put_aux(dsb, pos, 1, value);
-        break;
-    }
-}
-
 void mixieee32(float *src, float *dst, unsigned samples)
 {
     TRACE("%p - %p %d\n", src, dst, samples);
