@@ -38,6 +38,8 @@
 #include "ksmedia.h"
 
 #include "initguid.h"
+
+#include "mediaobj.h"
 #include "wingdi.h"
 #include "mmdeviceapi.h"
 #include "audioclient.h"
@@ -171,10 +173,10 @@ static void IDirectSound8_test(LPDIRECTSOUND8 dso, BOOL initialized,
     if (rc==DS_OK) {
         rc=IDirectSound8_GetSpeakerConfig(dso,&new_speaker_config);
         ok(rc==DS_OK,"IDirectSound8_GetSpeakerConfig() failed: %08x\n", rc);
-        if (rc==DS_OK && speaker_config!=new_speaker_config)
+        if (rc==DS_OK && speaker_config!=new_speaker_config && ref_speaker_config!=new_speaker_config)
                trace("IDirectSound8_GetSpeakerConfig() failed to set speaker "
-               "config: expected 0x%08x, got 0x%08x\n",
-               speaker_config,new_speaker_config);
+               "config: expected 0x%08x or 0x%08x, got 0x%08x\n",
+               speaker_config,ref_speaker_config,new_speaker_config);
         IDirectSound8_SetSpeakerConfig(dso,ref_speaker_config);
     }
 
@@ -790,7 +792,7 @@ static HRESULT test_secondary8(LPGUID lpGuid)
             wfxe.Format.cbSize = sizeof(wfxe) - sizeof(wfx) + 1;
 
             rc=IDirectSound_CreateSoundBuffer(dso,&bufdesc,&secondary,NULL);
-            ok(((rc==DSERR_CONTROLUNAVAIL || rc==DSERR_INVALIDCALL /* 2003 */) && !secondary)
+            ok(((rc==DSERR_CONTROLUNAVAIL || rc==DSERR_INVALIDCALL /* 2003 */ || rc==DSERR_INVALIDPARAM) && !secondary)
                 || rc==DS_OK /* driver dependent? */,
                 "IDirectSound_CreateSoundBuffer() returned: %08x %p\n",
                 rc, secondary);
@@ -814,7 +816,7 @@ static HRESULT test_secondary8(LPGUID lpGuid)
 
             wfxe.Format.cbSize = sizeof(wfxe);
             rc=IDirectSound_CreateSoundBuffer(dso,&bufdesc,&secondary,NULL);
-            ok((rc==DSERR_CONTROLUNAVAIL || rc==DSERR_INVALIDCALL || rc==E_INVALIDARG) && !secondary,
+            ok((rc==DSERR_CONTROLUNAVAIL || rc==DSERR_INVALIDCALL || rc==DSERR_INVALIDPARAM) && !secondary,
                 "IDirectSound_CreateSoundBuffer() returned: %08x %p\n",
                 rc, secondary);
             if (secondary)
@@ -836,7 +838,7 @@ static HRESULT test_secondary8(LPGUID lpGuid)
 
             wfxe.Format.cbSize = sizeof(wfxe) + 1;
             rc=IDirectSound_CreateSoundBuffer(dso,&bufdesc,&secondary,NULL);
-            ok(((rc==DSERR_CONTROLUNAVAIL || rc==DSERR_INVALIDCALL /* 2003 */) && !secondary)
+            ok(((rc==DSERR_CONTROLUNAVAIL || rc==DSERR_INVALIDCALL /* 2003 */ || rc==DSERR_INVALIDPARAM) && !secondary)
                 || rc==DS_OK /* driver dependent? */,
                 "IDirectSound_CreateSoundBuffer() returned: %08x %p\n",
                 rc, secondary);
@@ -1171,6 +1173,208 @@ static void test_COM(void)
     while (IUnknown_Release(unk));
 }
 
+static void test_effects(void)
+{
+    HRESULT rc;
+    LPDIRECTSOUND8 dso;
+    LPDIRECTSOUNDBUFFER primary, secondary;
+    LPDIRECTSOUNDBUFFER8 secondary8;
+    DSBUFFERDESC bufdesc;
+    WAVEFORMATEX wfx;
+    DSEFFECTDESC effects[2];
+    DWORD resultcodes[2];
+
+    /* Create a DirectSound8 object */
+    rc=pDirectSoundCreate8(NULL,&dso,NULL);
+    ok(rc==DS_OK||rc==DSERR_NODRIVER,"DirectSoundCreate8() failed: %08x\n",rc);
+
+    if (rc!=DS_OK)
+        return;
+
+    rc=IDirectSound8_SetCooperativeLevel(dso,get_hwnd(),DSSCL_PRIORITY);
+    ok(rc==DS_OK,"IDirectSound8_SetCooperativeLevel() failed: %08x\n", rc);
+    if (rc!=DS_OK) {
+        IDirectSound_Release(dso);
+        return;
+    }
+
+    primary=NULL;
+    ZeroMemory(&bufdesc, sizeof(bufdesc));
+    bufdesc.dwSize=sizeof(bufdesc);
+    bufdesc.dwFlags=DSBCAPS_PRIMARYBUFFER;
+    rc=IDirectSound8_CreateSoundBuffer(dso,&bufdesc,&primary,NULL);
+    ok((rc==DS_OK && primary!=NULL),
+       "IDirectSound8_CreateSoundBuffer() failed to create a primary buffer: "
+       "%08x\n",rc);
+    if (rc==DS_OK) {
+        init_format(&wfx,WAVE_FORMAT_PCM,11025,8,1);
+        ZeroMemory(&bufdesc, sizeof(bufdesc));
+        bufdesc.dwSize=sizeof(bufdesc);
+        bufdesc.dwFlags=0;
+        bufdesc.dwBufferBytes=align(wfx.nAvgBytesPerSec*BUFFER_LEN/1000,
+                                    wfx.nBlockAlign);
+        bufdesc.lpwfxFormat=&wfx;
+
+        ZeroMemory(effects, sizeof(effects));
+        effects[0].dwSize=sizeof(effects[0]);
+        effects[0].guidDSFXClass=GUID_DSFX_STANDARD_ECHO;
+        effects[1].dwSize=sizeof(effects[1]);
+        effects[1].guidDSFXClass=GUID_NULL;
+
+        secondary=NULL;
+        rc=IDirectSound8_CreateSoundBuffer(dso,&bufdesc,&secondary,NULL);
+        ok(rc==DS_OK && secondary!=NULL,
+           "IDirectSound8_CreateSoundBuffer() failed to create a secondary "
+           "buffer: %08x\n",rc);
+
+        /* Call SetFX on buffer without DSBCAPS_CTRLFX */
+        if (rc==DS_OK && secondary!=NULL) {
+            secondary8=NULL;
+            rc=IDirectSoundBuffer_QueryInterface(secondary,&IID_IDirectSoundBuffer8,(LPVOID*)&secondary8);
+            ok(rc==DS_OK,"IDirectSoundBuffer_QueryInterface(IID_IDirectSoundBuffer8) failed: %08x\n", rc);
+
+            if (rc==DS_OK && secondary8) {
+                rc=IDirectSoundBuffer8_SetFX(secondary8,1,effects,resultcodes);
+                ok(rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_SetFX() "
+                "should have returned DSERR_CONTROLUNAVAIL, returned: %08x\n", rc);
+
+                IDirectSoundBuffer8_Release(secondary8);
+            }
+            IDirectSoundBuffer_Release(secondary);
+        }
+
+        secondary=NULL;
+        bufdesc.dwFlags=DSBCAPS_CTRLFX;
+        rc=IDirectSound8_CreateSoundBuffer(dso,&bufdesc,&secondary,NULL);
+        ok(rc==DS_OK && secondary!=NULL,
+           "IDirectSound8_CreateSoundBuffer() failed to create a secondary "
+           "buffer: %08x\n",rc);
+
+        if (rc==DS_OK) {
+            secondary8=NULL;
+            rc=IDirectSoundBuffer_QueryInterface(secondary,&IID_IDirectSoundBuffer8,(LPVOID*)&secondary8);
+            ok(rc==DS_OK,"IDirectSoundBuffer_QueryInterface(IID_IDirectSoundBuffer8) failed: %08x\n", rc);
+
+            if (rc==DS_OK && secondary8) {
+                LPVOID ptr1,ptr2;
+                DWORD bytes1,bytes2;
+                IUnknown* obj = NULL;
+                HRESULT rc2;
+
+                /* Call SetFX with dwEffectsCount > 0 and pDSFXDesc == NULL */
+                rc=IDirectSoundBuffer8_SetFX(secondary8,1,NULL,NULL);
+                ok(rc==E_INVALIDARG||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_SetFX() "
+                "should have returned E_INVALIDARG, returned: %08x\n", rc);
+
+                /* Call SetFX with dwEffectsCount == 0 and pDSFXDesc != NULL */
+                rc=IDirectSoundBuffer8_SetFX(secondary8,0,effects,NULL);
+                ok(rc==E_INVALIDARG||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_SetFX() "
+                "should have returned E_INVALIDARG, returned: %08x\n", rc);
+
+                /* Call SetFX with dwEffectsCount == 0 and pdwResultCodes != NULL */
+                rc=IDirectSoundBuffer8_SetFX(secondary8,0,NULL,resultcodes);
+                ok(rc==E_INVALIDARG||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_SetFX() "
+                "should have returned E_INVALIDARG, returned: %08x\n", rc);
+
+                rc=IDirectSoundBuffer8_Lock(secondary8,0,0,&ptr1,&bytes1,&ptr2,&bytes2,DSBLOCK_ENTIREBUFFER);
+                ok(rc==DS_OK,"IDirectSoundBuffer8_Lock() failed: %08x\n",rc);
+
+                if (rc==DS_OK) {
+                    /* Call SetFX when buffer is locked */
+                    rc=IDirectSoundBuffer8_SetFX(secondary8,1,effects,resultcodes);
+                    ok(rc==DSERR_INVALIDCALL||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_SetFX() "
+                    "should have returned DSERR_INVALIDCALL, returned: %08x\n", rc);
+
+                    rc=IDirectSoundBuffer8_Unlock(secondary8,ptr1,bytes1,ptr2,bytes2);
+                    ok(rc==DS_OK,"IDirectSoundBuffer8_Unlock() failed: %08x\n",rc);
+                }
+
+                rc=IDirectSoundBuffer8_Play(secondary8,0,0,DSBPLAY_LOOPING);
+                ok(rc==DS_OK,"IDirectSoundBuffer8_Play() failed: %08x\n",rc);
+
+                if (rc==DS_OK) {
+                    /* Call SetFX when buffer is playing */
+                    rc=IDirectSoundBuffer8_SetFX(secondary8,1,effects,resultcodes);
+                    ok(rc==DSERR_INVALIDCALL||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_SetFX() "
+                    "should have returned DSERR_INVALIDCALL, returned: %08x\n", rc);
+
+                    rc=IDirectSoundBuffer8_Stop(secondary8);
+                    ok(rc==DS_OK,"IDirectSoundBuffer8_Stop() failed: %08x\n",rc);
+                }
+
+                /* Call SetFX with non-existent filter */
+                rc=IDirectSoundBuffer8_SetFX(secondary8,1,&effects[1],resultcodes);
+                ok(rc==REGDB_E_CLASSNOTREG||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_SetFX(GUID_NULL) "
+                    "should have returned REGDB_E_CLASSNOTREG, returned: %08x\n",rc);
+                if (rc!=DSERR_CONTROLUNAVAIL) {
+                    ok(resultcodes[0]==DSFXR_UNKNOWN,"result code == %08x, expected DSFXR_UNKNOWN\n",resultcodes[0]);
+                }
+
+                /* Call SetFX with standard echo */
+                rc2=IDirectSoundBuffer8_SetFX(secondary8,1,&effects[0],resultcodes);
+                ok(rc2==DS_OK||rc2==REGDB_E_CLASSNOTREG||rc2==DSERR_CONTROLUNAVAIL,
+                   "IDirectSoundBuffer8_SetFX(GUID_DSFX_STANDARD_ECHO) failed: %08x\n",rc);
+                if (rc2!=DSERR_CONTROLUNAVAIL) {
+                    ok(resultcodes[0]==DSFXR_UNKNOWN||resultcodes[0]==DSFXR_LOCHARDWARE||resultcodes[0]==DSFXR_LOCSOFTWARE,
+                        "resultcode == %08x, expected DSFXR_UNKNOWN, DSFXR_LOCHARDWARE, or DSFXR_LOCSOFTWARE\n",resultcodes[0]);
+                }
+
+                /* Call GetObjectInPath for out-of-bounds DMO */
+                rc=IDirectSoundBuffer8_GetObjectInPath(secondary8,&GUID_All_Objects,2,&IID_IMediaObject,(void**)&obj);
+                ok(rc==DSERR_OBJECTNOTFOUND||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_GetObjectInPath() "
+                "should have returned DSERR_OBJECTNOTFOUND, returned: %08x\n",rc);
+
+                /* Call GetObjectInPath with NULL ppObject */
+                rc=IDirectSoundBuffer8_GetObjectInPath(secondary8,&GUID_All_Objects,0,&IID_IMediaObject,NULL);
+                ok(rc==E_INVALIDARG||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_GetObjectInPath() "
+                "should have returned E_INVALIDARG, returned: %08x\n",rc);
+
+                /* Call GetObjectInPath for unsupported interface */
+                rc=IDirectSoundBuffer8_GetObjectInPath(secondary8,&GUID_All_Objects,0,&GUID_NULL,(void**)&obj);
+                ok(rc==E_NOINTERFACE||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_GetObjectInPath() "
+                "should have returned E_NOINTERFACE, returned: %08x\n",rc);
+
+                /* Call GetObjectInPath for unloaded DMO */
+                rc=IDirectSoundBuffer8_GetObjectInPath(secondary8,&GUID_NULL,0,&IID_IMediaObject,(void**)&obj);
+                ok(rc==DSERR_OBJECTNOTFOUND||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_GetObjectInPath() "
+                "should have returned DSERR_OBJECTNOTFOUND, returned: %08x\n",rc);
+
+                /* Call GetObjectInPath for first DMO */
+                obj=NULL;
+                rc=IDirectSoundBuffer8_GetObjectInPath(secondary8,&GUID_All_Objects,0,&IID_IMediaObject,(void**)&obj);
+                if (rc2==DS_OK) {
+                    ok(rc==DS_OK||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_GetObjectInPath() "
+                    "should have returned DS_OK, returned: %08x\n",rc);
+                    if (obj) IUnknown_Release(obj);
+                } else {
+                    ok(rc==DSERR_OBJECTNOTFOUND||rc==DSERR_CONTROLUNAVAIL,"IDirectSoundBuffer8_GetObjectInPath() "
+                    "should have returned DSERR_OBJECTNOTFOUND, returned: %08x\n",rc);
+                }
+
+                /* Call SetFX with one real filter and one fake one */
+                rc=IDirectSoundBuffer8_SetFX(secondary8,2,effects,resultcodes);
+                ok(rc==REGDB_E_CLASSNOTREG||rc==DSERR_CONTROLUNAVAIL,
+                   "IDirectSoundBuffer8_SetFX(GUID_DSFX_STANDARD_ECHO, GUID_NULL) "
+                    "should have returned REGDB_E_CLASSNOTREG, returned: %08x\n",rc);
+                if (rc!=DSERR_CONTROLUNAVAIL) {
+                    ok(resultcodes[0]==DSFXR_PRESENT||resultcodes[0]==DSFXR_UNKNOWN,
+                        "resultcodes[0] == %08x, expected DSFXR_PRESENT or DSFXR_UNKNOWN\n",resultcodes[0]);
+                    ok(resultcodes[1]==DSFXR_UNKNOWN,
+                        "resultcodes[1] == %08x, expected DSFXR_UNKNOWN\n",resultcodes[1]);
+                }
+
+                IDirectSoundBuffer8_Release(secondary8);
+            }
+
+            IDirectSoundBuffer_Release(secondary);
+        }
+
+        IDirectSoundBuffer_Release(primary);
+    }
+
+    while (IDirectSound_Release(dso));
+}
+
 START_TEST(dsound8)
 {
     HMODULE hDsound;
@@ -1192,6 +1396,7 @@ START_TEST(dsound8)
             dsound8_tests();
             test_hw_buffers();
             test_first_device();
+            test_effects();
         }
         else
             skip("DirectSoundCreate8 missing - skipping all tests\n");
