@@ -28,13 +28,13 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(dwrite);
 
-extern const unsigned short wine_linebreak_table[];
-extern const unsigned short wine_scripts_table[];
+extern const unsigned short wine_linebreak_table[] DECLSPEC_HIDDEN;
+extern const unsigned short wine_scripts_table[] DECLSPEC_HIDDEN;
 
 struct dwritescript_properties {
     DWRITE_SCRIPT_PROPERTIES props;
     UINT32 scripttag;    /* OpenType script tag */
-    UINT32 scriptalttag; /* Version 2 tag, 0 is not defined */
+    UINT32 scriptalttag; /* Version 2 tag, 0 if not defined */
     BOOL is_complex;
     const struct scriptshaping_ops *ops;
 };
@@ -177,6 +177,33 @@ static const struct dwritescript_properties dwritescripts_properties[Script_Last
 };
 #undef _OT
 
+const char *debugstr_sa_script(UINT16 script)
+{
+    return script < Script_LastId ? debugstr_an((char*)&dwritescripts_properties[script].props.isoScriptCode, 4): "not defined";
+}
+
+/* system font falback configuration */
+static const WCHAR meiryoW[] = {'M','e','i','r','y','o',0};
+
+struct fallback_mapping {
+    DWRITE_UNICODE_RANGE range;
+    const WCHAR *family;
+};
+
+static const struct fallback_mapping fontfallback_neutral_data[] = {
+    { { 0x3000, 0x30ff }, meiryoW }, /* CJK Symbols and Punctuation, Hiragana, Katakana */
+    { { 0x31f0, 0x31ff }, meiryoW }, /* Katakana Phonetic Extensions */
+    { { 0x4e00, 0x9fff }, meiryoW }, /* CJK Unified Ideographs */
+};
+
+struct dwrite_fontfallback {
+    IDWriteFontFallback IDWriteFontFallback_iface;
+    IDWriteFactory3 *factory;
+    IDWriteFontCollection *systemcollection;
+    const struct fallback_mapping *mappings;
+    UINT32 count;
+};
+
 struct dwrite_numbersubstitution {
     IDWriteNumberSubstitution IDWriteNumberSubstitution_iface;
     LONG ref;
@@ -191,6 +218,11 @@ static inline struct dwrite_numbersubstitution *impl_from_IDWriteNumberSubstitut
     return CONTAINING_RECORD(iface, struct dwrite_numbersubstitution, IDWriteNumberSubstitution_iface);
 }
 
+static inline struct dwrite_fontfallback *impl_from_IDWriteFontFallback(IDWriteFontFallback *iface)
+{
+    return CONTAINING_RECORD(iface, struct dwrite_fontfallback, IDWriteFontFallback_iface);
+}
+
 static inline UINT32 decode_surrogate_pair(const WCHAR *str, UINT32 index, UINT32 end)
 {
     if (index < end-1 && IS_SURROGATE_PAIR(str[index], str[index+1])) {
@@ -203,53 +235,67 @@ static inline UINT32 decode_surrogate_pair(const WCHAR *str, UINT32 index, UINT3
 
 static inline UINT16 get_char_script(WCHAR c)
 {
-    UINT16 script = get_table_entry(wine_scripts_table, c);
-    if (script == Script_Unknown) {
-        WORD type;
-        if (GetStringTypeW(CT_CTYPE1, &c, 1, &type) && (type & C1_CNTRL))
-            script = Script_Common;
-    }
-    return script;
+    return get_table_entry(wine_scripts_table, c);
 }
 
-static HRESULT analyze_script(const WCHAR *text, UINT32 position, UINT32 len, IDWriteTextAnalysisSink *sink)
+static DWRITE_SCRIPT_ANALYSIS get_char_sa(WCHAR c)
 {
     DWRITE_SCRIPT_ANALYSIS sa;
-    UINT32 pos, i, length;
 
-    if (!len) return S_OK;
+    sa.script = get_char_script(c);
+    sa.shapes = iscntrlW(c) || c == 0x2028 /* LINE SEPARATOR */ || c == 0x2029 /* PARAGRAPH SEPARATOR */ ?
+        DWRITE_SCRIPT_SHAPES_NO_VISUAL : DWRITE_SCRIPT_SHAPES_DEFAULT;
+    return sa;
+}
 
-    sa.script = get_char_script(*text);
+static HRESULT analyze_script(const WCHAR *text, UINT32 position, UINT32 length, IDWriteTextAnalysisSink *sink)
+{
+    DWRITE_SCRIPT_ANALYSIS sa;
+    UINT32 pos, i, seq_length;
+
+    if (!length) return S_OK;
+
+    sa = get_char_sa(*text);
 
     pos = position;
-    length = 1;
+    seq_length = 1;
 
-    for (i = 1; i < len; i++)
+    for (i = 1; i < length; i++)
     {
-        UINT16 script = get_char_script(text[i]);
+        DWRITE_SCRIPT_ANALYSIS cur_sa = get_char_sa(text[i]);
 
         /* Unknown type is ignored when preceded or followed by another script */
-        if (sa.script == Script_Unknown) sa.script = script;
-        if (script == Script_Unknown && sa.script != Script_Common) script = sa.script;
-        /* this is a length of a sequence to be reported next */
-        if (sa.script == script) length++;
+        switch (sa.script) {
+        case Script_Unknown:
+            sa.script = cur_sa.script;
+            break;
+        case Script_Common:
+            if (cur_sa.script == Script_Unknown)
+                cur_sa.script = sa.script;
+            else if (cur_sa.script != Script_Common)
+                sa.script = cur_sa.script;
+            break;
+        default:
+            if (cur_sa.script == Script_Unknown || cur_sa.script == Script_Common)
+                cur_sa.script = sa.script;
+        }
 
-        if (sa.script != script)
-        {
+        /* this is a length of a sequence to be reported next */
+        if (sa.script == cur_sa.script && sa.shapes == cur_sa.shapes)
+            seq_length++;
+        else {
             HRESULT hr;
 
-            sa.shapes = sa.script != Script_Common ? DWRITE_SCRIPT_SHAPES_DEFAULT : DWRITE_SCRIPT_SHAPES_NO_VISUAL;
-            hr = IDWriteTextAnalysisSink_SetScriptAnalysis(sink, pos, length, &sa);
+            hr = IDWriteTextAnalysisSink_SetScriptAnalysis(sink, pos, seq_length, &sa);
             if (FAILED(hr)) return hr;
             pos = position + i;
-            length = 1;
-            sa.script = script;
+            seq_length = 1;
+            sa = cur_sa;
         }
     }
 
-    /* 1 length case or normal completion call */
-    sa.shapes = sa.script != Script_Common ? DWRITE_SCRIPT_SHAPES_DEFAULT : DWRITE_SCRIPT_SHAPES_NO_VISUAL;
-    return IDWriteTextAnalysisSink_SetScriptAnalysis(sink, pos, length, &sa);
+    /* one char length case or normal completion call */
+    return IDWriteTextAnalysisSink_SetScriptAnalysis(sink, pos, seq_length, &sa);
 }
 
 struct linebreaking_state {
@@ -326,6 +372,12 @@ static inline void set_break_condition(UINT32 pos, enum BreakConditionLocation l
     }
 }
 
+BOOL lb_is_newline_char(WCHAR ch)
+{
+    short c = get_table_entry(wine_linebreak_table, ch);
+    return c == b_LF || c == b_NL || c == b_CR || c == b_BK;
+}
+
 static HRESULT analyze_linebreaks(const WCHAR *text, UINT32 count, DWRITE_LINE_BREAKPOINT *breakpoints)
 {
     struct linebreaking_state state;
@@ -347,8 +399,8 @@ static HRESULT analyze_linebreaks(const WCHAR *text, UINT32 count, DWRITE_LINE_B
 
         breakpoints[i].breakConditionBefore = DWRITE_BREAK_CONDITION_CAN_BREAK;
         breakpoints[i].breakConditionAfter  = DWRITE_BREAK_CONDITION_CAN_BREAK;
-        breakpoints[i].isWhitespace = break_class[i] == b_BK || break_class[i] == b_ZW || break_class[i] == b_SP || isspaceW(text[i]);
-        breakpoints[i].isSoftHyphen = FALSE;
+        breakpoints[i].isWhitespace = !!isspaceW(text[i]);
+        breakpoints[i].isSoftHyphen = text[i] == 0x00ad /* Unicode Soft Hyphen */;
         breakpoints[i].padding = 0;
 
         /* LB1 - resolve some classes. TODO: use external algorithms for these classes. */
@@ -522,15 +574,19 @@ static HRESULT analyze_linebreaks(const WCHAR *text, UINT32 count, DWRITE_LINE_B
             case b_BB:
                 set_break_condition(i, BreakConditionAfter, DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, &state);
                 break;
-            /* LB21a */
+            /* LB21a, LB21b */
             case b_HL:
-                if (i < count-2)
+                /* LB21a */
+                if (i < count-1)
                     switch (break_class[i+1])
                     {
                     case b_HY:
                     case b_BA:
                         set_break_condition(i+1, BreakConditionAfter, DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, &state);
                     }
+                /* LB21b */
+                if (i > 0 && break_class[i-1] == b_SY)
+                    set_break_condition(i, BreakConditionBefore, DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, &state);
                 break;
             /* LB22 */
             case b_IN:
@@ -540,6 +596,7 @@ static HRESULT analyze_linebreaks(const WCHAR *text, UINT32 count, DWRITE_LINE_B
                     {
                         case b_AL:
                         case b_HL:
+                        case b_EX:
                         case b_ID:
                         case b_IN:
                         case b_NU:
@@ -750,14 +807,14 @@ static HRESULT WINAPI dwritetextanalyzer_AnalyzeBidi(IDWriteTextAnalyzer2 *iface
 {
     UINT8 *levels = NULL, *explicit = NULL;
     UINT8 baselevel, level, explicit_level;
+    UINT32 pos, i, seq_length;
     WCHAR *buff = NULL;
     const WCHAR *text;
-    UINT32 pos, i;
     HRESULT hr;
 
     TRACE("(%p %u %u %p)\n", source, position, length, sink);
 
-    if (length == 0)
+    if (!length)
         return S_OK;
 
     hr = get_text_source_ptr(source, position, length, &text, &buff);
@@ -779,20 +836,25 @@ static HRESULT WINAPI dwritetextanalyzer_AnalyzeBidi(IDWriteTextAnalyzer2 *iface
 
     level = levels[0];
     explicit_level = explicit[0];
-    pos = 0;
+    pos = position;
+    seq_length = 1;
+
     for (i = 1; i < length; i++) {
-        if (levels[i] != level || explicit[i] != explicit_level) {
-            hr = IDWriteTextAnalysisSink_SetBidiLevel(sink, pos, i - pos, explicit_level, level);
+        if (levels[i] == level && explicit[i] == explicit_level)
+            seq_length++;
+        else {
+            hr = IDWriteTextAnalysisSink_SetBidiLevel(sink, pos, seq_length, explicit_level, level);
             if (FAILED(hr))
-                break;
+                goto done;
+
+            pos += seq_length;
+            seq_length = 1;
             level = levels[i];
             explicit_level = explicit[i];
-            pos = i;
         }
-
-        if (i == length - 1)
-            hr = IDWriteTextAnalysisSink_SetBidiLevel(sink, pos, length - pos, explicit_level, level);
     }
+    /* one char length case or normal completion call */
+    hr = IDWriteTextAnalysisSink_SetBidiLevel(sink, pos, seq_length, explicit_level, level);
 
 done:
     heap_free(explicit);
@@ -1049,15 +1111,18 @@ static HRESULT WINAPI dwritetextanalyzer_GetGlyphPlacements(IDWriteTextAnalyzer2
 
     IDWriteFontFace_GetMetrics(fontface, &metrics);
     for (i = 0; i < glyph_count; i++) {
-        INT32 a;
+        if (glyph_props[i].isZeroWidthSpace)
+            advances[i] = 0.0f;
+        else {
+            INT32 a;
 
-        hr = IDWriteFontFace1_GetDesignGlyphAdvances(fontface1, 1, &glyphs[i], &a, is_sideways);
-        if (FAILED(hr))
-            a = 0;
-
-        advances[i] = get_scaled_advance_width(a, emSize, &metrics);
-        offsets[i].advanceOffset = 0.0;
-        offsets[i].ascenderOffset = 0.0;
+            hr = IDWriteFontFace1_GetDesignGlyphAdvances(fontface1, 1, &glyphs[i], &a, is_sideways);
+            if (FAILED(hr))
+                a = 0;
+            advances[i] = get_scaled_advance_width(a, emSize, &metrics);
+        }
+        offsets[i].advanceOffset = 0.0f;
+        offsets[i].ascenderOffset = 0.0f;
     }
 
     /* FIXME: actually apply features */
@@ -1105,11 +1170,11 @@ static HRESULT WINAPI dwritetextanalyzer_GetGdiCompatibleGlyphPlacements(IDWrite
         hr = IDWriteFontFace1_GetGdiCompatibleGlyphAdvances(fontface1, emSize, ppdip,
             transform, use_gdi_natural, is_sideways, 1, &glyphs[i], &a);
         if (FAILED(hr))
-            advances[i] = 0.0;
+            advances[i] = 0.0f;
         else
             advances[i] = floorf(a * emSize * ppdip / metrics.designUnitsPerEm + 0.5f) / ppdip;
-        offsets[i].advanceOffset = 0.0;
-        offsets[i].ascenderOffset = 0.0;
+        offsets[i].advanceOffset = 0.0f;
+        offsets[i].ascenderOffset = 0.0f;
     }
 
     /* FIXME: actually apply features */
@@ -1120,7 +1185,7 @@ static HRESULT WINAPI dwritetextanalyzer_GetGdiCompatibleGlyphPlacements(IDWrite
 
 static inline FLOAT get_cluster_advance(const FLOAT *advances, UINT32 start, UINT32 end)
 {
-    FLOAT advance = 0.0;
+    FLOAT advance = 0.0f;
     for (; start < end; start++)
         advance += advances[start];
     return advance;
@@ -1130,9 +1195,9 @@ static void apply_single_glyph_spacing(FLOAT leading_spacing, FLOAT trailing_spa
     FLOAT min_advance_width, UINT32 g, FLOAT const *advances, DWRITE_GLYPH_OFFSET const *offsets,
     DWRITE_SHAPING_GLYPH_PROPERTIES const *props, FLOAT *modified_advances, DWRITE_GLYPH_OFFSET *modified_offsets)
 {
-    BOOL reduced = leading_spacing < 0.0 || trailing_spacing < 0.0;
+    BOOL reduced = leading_spacing < 0.0f || trailing_spacing < 0.0f;
     FLOAT advance = advances[g];
-    FLOAT origin = 0.0;
+    FLOAT origin = 0.0f;
 
     if (props[g].isZeroWidthSpace) {
         modified_advances[g] = advances[g];
@@ -1141,32 +1206,32 @@ static void apply_single_glyph_spacing(FLOAT leading_spacing, FLOAT trailing_spa
     }
 
     /* first apply negative spacing and check if we hit minimum width */
-    if (leading_spacing < 0.0) {
+    if (leading_spacing < 0.0f) {
         advance += leading_spacing;
         origin -= leading_spacing;
     }
-    if (trailing_spacing < 0.0)
+    if (trailing_spacing < 0.0f)
         advance += trailing_spacing;
 
     if (advance < min_advance_width) {
-        FLOAT half = (min_advance_width - advance) / 2.0;
+        FLOAT half = (min_advance_width - advance) / 2.0f;
 
         if (!reduced)
             origin -= half;
-        else if (leading_spacing < 0.0 && trailing_spacing < 0.0)
+        else if (leading_spacing < 0.0f && trailing_spacing < 0.0f)
             origin -= half;
-        else if (leading_spacing < 0.0)
+        else if (leading_spacing < 0.0f)
             origin -= min_advance_width - advance;
 
         advance = min_advance_width;
     }
 
     /* now apply positive spacing adjustments */
-    if (leading_spacing > 0.0) {
+    if (leading_spacing > 0.0f) {
         advance += leading_spacing;
         origin -= leading_spacing;
     }
-    if (trailing_spacing > 0.0)
+    if (trailing_spacing > 0.0f)
         advance += trailing_spacing;
 
     modified_advances[g] = advance;
@@ -1180,21 +1245,21 @@ static void apply_cluster_spacing(FLOAT leading_spacing, FLOAT trailing_spacing,
     UINT32 start, UINT32 end, FLOAT const *advances, DWRITE_GLYPH_OFFSET const *offsets,
     FLOAT *modified_advances, DWRITE_GLYPH_OFFSET *modified_offsets)
 {
-    BOOL reduced = leading_spacing < 0.0 || trailing_spacing < 0.0;
+    BOOL reduced = leading_spacing < 0.0f || trailing_spacing < 0.0f;
     FLOAT advance = get_cluster_advance(advances, start, end);
-    FLOAT origin = 0.0;
+    FLOAT origin = 0.0f;
     UINT16 g;
 
     modified_advances[start] = advances[start];
     modified_advances[end-1] = advances[end-1];
 
     /* first apply negative spacing and check if we hit minimum width */
-    if (leading_spacing < 0.0) {
+    if (leading_spacing < 0.0f) {
         advance += leading_spacing;
         modified_advances[start] += leading_spacing;
         origin -= leading_spacing;
     }
-    if (trailing_spacing < 0.0) {
+    if (trailing_spacing < 0.0f) {
         advance += trailing_spacing;
         modified_advances[end-1] += trailing_spacing;
     }
@@ -1209,12 +1274,12 @@ static void apply_cluster_spacing(FLOAT leading_spacing, FLOAT trailing_spacing,
             modified_advances[start] += half;
             modified_advances[end-1] += half;
         }
-        else if (leading_spacing < 0.0 && trailing_spacing < 0.0) {
+        else if (leading_spacing < 0.0f && trailing_spacing < 0.0f) {
             origin -= half;
             modified_advances[start] += half;
             modified_advances[end-1] += half;
         }
-        else if (leading_spacing < 0.0) {
+        else if (leading_spacing < 0.0f) {
             origin -= advance;
             modified_advances[start] += advance;
         }
@@ -1223,11 +1288,11 @@ static void apply_cluster_spacing(FLOAT leading_spacing, FLOAT trailing_spacing,
     }
 
     /* now apply positive spacing adjustments */
-    if (leading_spacing > 0.0) {
+    if (leading_spacing > 0.0f) {
         modified_advances[start] += leading_spacing;
         origin -= leading_spacing;
     }
-    if (trailing_spacing > 0.0)
+    if (trailing_spacing > 0.0f)
         modified_advances[end-1] += trailing_spacing;
 
     for (g = start; g < end; g++) {
@@ -1273,15 +1338,15 @@ static inline UINT32 get_cluster_length(UINT16 const *clustermap, UINT32 start,
       If it's the case advance width is incremented up to minimum value.
 
       Important part is the direction in which this increment is applied;
-      it depends on from which directions total cluster advance was trimmed
+      it depends on direction from which total cluster advance was trimmed
       at step 1. So it could be incremented from leading, trailing, or both
       sides. When applied to both sides, each side gets half of difference
-      that bring advance to minimum width.
+      that brings advance to minimum width.
 
    3. Positive adjustments
 
-      After minimum width rule was applied, positive spacing is applied in same
-      way as negative ones on step 1.
+      After minimum width rule was applied, positive spacing is applied in the same
+      way as negative one on step 1.
 
    Glyph offset for leading glyph is adjusted too in a way that glyph origin
    keeps its position in coordinate system where initial advance width is counted
@@ -1305,13 +1370,13 @@ static HRESULT WINAPI dwritetextanalyzer1_ApplyCharacterSpacing(IDWriteTextAnaly
     TRACE("(%.2f %.2f %.2f %u %u %p %p %p %p %p %p)\n", leading_spacing, trailing_spacing, min_advance_width,
         len, glyph_count, clustermap, advances, offsets, props, modified_advances, modified_offsets);
 
-    if (min_advance_width < 0.0) {
+    if (min_advance_width < 0.0f) {
         memset(modified_advances, 0, glyph_count*sizeof(*modified_advances));
         return E_INVALIDARG;
     }
 
     /* minimum advance is not applied if no adjustments were made */
-    if (leading_spacing == 0.0 && trailing_spacing == 0.0) {
+    if (leading_spacing == 0.0f && trailing_spacing == 0.0f) {
         memmove(modified_advances, advances, glyph_count*sizeof(*advances));
         memmove(modified_offsets, offsets, glyph_count*sizeof(*offsets));
         return S_OK;
@@ -1469,10 +1534,10 @@ static HRESULT WINAPI dwritetextanalyzer2_GetGlyphOrientationTransform(IDWriteTe
     DWRITE_GLYPH_ORIENTATION_ANGLE angle, BOOL is_sideways, FLOAT originX, FLOAT originY, DWRITE_MATRIX *m)
 {
     static const DWRITE_MATRIX transforms[] = {
-        {  1.0,  0.0,  0.0,  1.0, 0.0, 0.0 },
-        {  0.0,  1.0, -1.0,  0.0, 0.0, 0.0 },
-        { -1.0,  0.0,  0.0, -1.0, 0.0, 0.0 },
-        {  0.0, -1.0,  1.0,  0.0, 0.0, 0.0 }
+        {  1.0f,  0.0f,  0.0f,  1.0f, 0.0f, 0.0f },
+        {  0.0f,  1.0f, -1.0f,  0.0f, 0.0f, 0.0f },
+        { -1.0f,  0.0f,  0.0f, -1.0f, 0.0f, 0.0f },
+        {  0.0f, -1.0f,  1.0f,  0.0f, 0.0f, 0.0f }
     };
 
     TRACE("(%d %d %.2f %.2f %p)\n", angle, is_sideways, originX, originY, m);
@@ -1506,7 +1571,7 @@ static HRESULT WINAPI dwritetextanalyzer2_GetGlyphOrientationTransform(IDWriteTe
 
     /* shift components represent transform necessary to get from original point to
        rotated one in new coordinate system */
-    if ((originX != 0.0 || originY != 0.0) && angle != DWRITE_GLYPH_ORIENTATION_ANGLE_0_DEGREES) {
+    if ((originX != 0.0f || originY != 0.0f) && angle != DWRITE_GLYPH_ORIENTATION_ANGLE_0_DEGREES) {
         m->dx = originX - (m->m11 * originX + m->m21 * originY);
         m->dy = originY - (m->m12 * originX + m->m22 * originY);
     }
@@ -1661,3 +1726,234 @@ HRESULT create_numbersubstitution(DWRITE_NUMBER_SUBSTITUTION_METHOD method, cons
     *ret = &substitution->IDWriteNumberSubstitution_iface;
     return S_OK;
 }
+
+/* IDWriteFontFallback */
+static HRESULT WINAPI fontfallback_QueryInterface(IDWriteFontFallback *iface, REFIID riid, void **obj)
+{
+    struct dwrite_fontfallback *fallback = impl_from_IDWriteFontFallback(iface);
+
+    TRACE("(%p)->(%s %p)\n", fallback, debugstr_guid(riid), obj);
+
+    if (IsEqualIID(riid, &IID_IDWriteFontFallback) || IsEqualIID(riid, &IID_IUnknown)) {
+        *obj = iface;
+        IDWriteFontFallback_AddRef(iface);
+        return S_OK;
+    }
+
+    *obj = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI fontfallback_AddRef(IDWriteFontFallback *iface)
+{
+    struct dwrite_fontfallback *fallback = impl_from_IDWriteFontFallback(iface);
+    TRACE("(%p)\n", fallback);
+    return IDWriteFactory3_AddRef(fallback->factory);
+}
+
+static ULONG WINAPI fontfallback_Release(IDWriteFontFallback *iface)
+{
+    struct dwrite_fontfallback *fallback = impl_from_IDWriteFontFallback(iface);
+    TRACE("(%p)\n", fallback);
+    return IDWriteFactory3_Release(fallback->factory);
+}
+
+static int compare_fallback_mapping(const void *a, const void *b)
+{
+    UINT32 ch = *(UINT32*)a;
+    struct fallback_mapping *mapping = (struct fallback_mapping*)b;
+
+    if (ch > mapping->range.last)
+        return 1;
+    else if (ch < mapping->range.first)
+        return -1;
+    else
+        return 0;
+}
+
+static const struct fallback_mapping *find_fallback_mapping(struct dwrite_fontfallback *fallback, UINT32 ch)
+{
+    return bsearch(&ch, fallback->mappings, fallback->count, sizeof(*fallback->mappings),
+        compare_fallback_mapping);
+}
+
+HRESULT create_matching_font(IDWriteFontCollection *collection, const WCHAR *name,
+    DWRITE_FONT_WEIGHT weight, DWRITE_FONT_STYLE style, DWRITE_FONT_STRETCH stretch, IDWriteFont **font)
+{
+    IDWriteFontFamily *family;
+    BOOL exists = FALSE;
+    HRESULT hr;
+    UINT32 i;
+
+    *font = NULL;
+
+    hr = IDWriteFontCollection_FindFamilyName(collection, name, &i, &exists);
+    if (FAILED(hr))
+        return hr;
+
+    if (!exists)
+        return E_FAIL;
+
+    hr = IDWriteFontCollection_GetFontFamily(collection, i, &family);
+    if (FAILED(hr))
+        return hr;
+
+    hr = IDWriteFontFamily_GetFirstMatchingFont(family, weight, stretch, style, font);
+    IDWriteFontFamily_Release(family);
+    return hr;
+}
+
+static HRESULT fallback_map_characters(IDWriteFont *font, const WCHAR *text, UINT32 length, UINT32 *mapped_length)
+{
+    HRESULT hr = S_OK;
+    UINT32 i;
+
+    for (i = 0; i < length; i++) {
+        UINT16 script = get_char_script(text[i]);
+        BOOL exists;
+
+        if (script == Script_Unknown || script == Script_Common) {
+            ++*mapped_length;
+            continue;
+        }
+
+        /* stop on first unsupported character */
+        exists = FALSE;
+        hr = IDWriteFont_HasCharacter(font, text[i], &exists);
+        if (hr == S_OK && exists)
+            ++*mapped_length;
+        else
+            break;
+    }
+
+    return hr;
+}
+
+static HRESULT fallback_get_fallback_font(struct dwrite_fontfallback *fallback, const WCHAR *text, UINT32 length,
+    DWRITE_FONT_WEIGHT weight, DWRITE_FONT_STYLE style, DWRITE_FONT_STRETCH stretch, UINT32 *mapped_length, IDWriteFont **mapped_font)
+{
+    const struct fallback_mapping *mapping;
+    HRESULT hr;
+
+    mapping = find_fallback_mapping(fallback, text[0]);
+    if (!mapping) {
+        WARN("no mapping for 0x%x\n", text[0]);
+        return E_FAIL;
+    }
+
+    /* now let's see what fallback can handle */
+    hr = create_matching_font(fallback->systemcollection, mapping->family, weight, style, stretch, mapped_font);
+    if (FAILED(hr)) {
+        WARN("failed to create fallback font %s for range [0x%x,0x%x], 0x%08x\n", debugstr_w(mapping->family),
+	    mapping->range.first, mapping->range.last, hr);
+        return hr;
+    }
+
+    hr = fallback_map_characters(*mapped_font, text, length, mapped_length);
+    if (FAILED(hr))
+        WARN("mapping with fallback font %s failed, 0x%08x\n", debugstr_w(mapping->family), hr);
+
+    if (!*mapped_length) {
+        IDWriteFont_Release(*mapped_font);
+        *mapped_font = NULL;
+    }
+
+    return *mapped_length ? S_OK : E_FAIL;
+}
+
+static HRESULT WINAPI fontfallback_MapCharacters(IDWriteFontFallback *iface, IDWriteTextAnalysisSource *source,
+    UINT32 position, UINT32 length, IDWriteFontCollection *basecollection, const WCHAR *basefamily,
+    DWRITE_FONT_WEIGHT weight, DWRITE_FONT_STYLE style, DWRITE_FONT_STRETCH stretch, UINT32 *mapped_length,
+    IDWriteFont **ret_font, FLOAT *scale)
+{
+    struct dwrite_fontfallback *fallback = impl_from_IDWriteFontFallback(iface);
+    WCHAR *buff = NULL;
+    const WCHAR *text;
+    HRESULT hr;
+
+    TRACE("(%p)->(%p %u %u %p, %s, %u, %u, %u, %p, %p, %p)\n", fallback, source, position, length,
+        basecollection, debugstr_w(basefamily), weight, style, stretch, mapped_length, ret_font, scale);
+
+    *mapped_length = 0;
+    *scale = 1.0f;
+    *ret_font = NULL;
+
+    if (!source)
+        return E_INVALIDARG;
+
+    if (length == 0)
+        return S_OK;
+
+    if (!basecollection)
+        basecollection = fallback->systemcollection;
+
+    hr = get_text_source_ptr(source, position, length, &text, &buff);
+    if (FAILED(hr))
+        goto done;
+
+    if (basefamily && *basefamily) {
+        hr = create_matching_font(basecollection, basefamily, weight, style, stretch, ret_font);
+        if (FAILED(hr))
+            goto done;
+
+        hr = fallback_map_characters(*ret_font, text, length, mapped_length);
+        if (FAILED(hr))
+            goto done;
+    }
+
+    if (!*mapped_length) {
+        IDWriteFont *mapped_font;
+
+        hr = fallback_get_fallback_font(fallback, text, length, weight, style, stretch, mapped_length, &mapped_font);
+        if (FAILED(hr)) {
+            /* fallback wasn't found, keep base font if any, so we can get at least some visual output */
+            if (*ret_font) {
+                *mapped_length = length;
+                hr = S_OK;
+            }
+        }
+        else {
+            if (*ret_font)
+                IDWriteFont_Release(*ret_font);
+            *ret_font = mapped_font;
+        }
+    }
+
+done:
+    heap_free(buff);
+    return hr;
+}
+
+static const IDWriteFontFallbackVtbl fontfallbackvtbl = {
+    fontfallback_QueryInterface,
+    fontfallback_AddRef,
+    fontfallback_Release,
+    fontfallback_MapCharacters
+};
+
+HRESULT create_system_fontfallback(IDWriteFactory3 *factory, IDWriteFontFallback **ret)
+{
+    struct dwrite_fontfallback *fallback;
+
+    *ret = NULL;
+
+    fallback = heap_alloc(sizeof(*fallback));
+    if (!fallback)
+        return E_OUTOFMEMORY;
+
+    fallback->IDWriteFontFallback_iface.lpVtbl = &fontfallbackvtbl;
+    fallback->factory = factory;
+    fallback->mappings = fontfallback_neutral_data;
+    fallback->count = sizeof(fontfallback_neutral_data)/sizeof(fontfallback_neutral_data[0]);
+    IDWriteFactory2_GetSystemFontCollection((IDWriteFactory2*)fallback->factory, &fallback->systemcollection, FALSE);
+
+    *ret = &fallback->IDWriteFontFallback_iface;
+    return S_OK;
+}
+
+void release_system_fontfallback(IDWriteFontFallback *iface)
+{
+    struct dwrite_fontfallback *fallback = impl_from_IDWriteFontFallback(iface);
+    IDWriteFontCollection_Release(fallback->systemcollection);
+    heap_free(fallback);
+}
