@@ -188,6 +188,7 @@ static void test_create_surface(void)
     DXGI_SURFACE_DESC desc;
     IDXGISurface *surface;
     IDXGIDevice *device;
+    IUnknown *surface1;
     IUnknown *texture;
     ULONG refcount;
     HRESULT hr;
@@ -216,6 +217,11 @@ static void test_create_surface(void)
             "Surface should implement ID3D11Texture2D.\n");
     if (SUCCEEDED(hr)) IUnknown_Release(texture);
 
+    hr = IDXGISurface_QueryInterface(surface, &IID_IDXGISurface1, (void **)&surface1);
+    ok(SUCCEEDED(hr) || broken(hr == E_NOINTERFACE) /* Not available on all Windows versions. */,
+            "Surface should implement IDXGISurface1.\n");
+    if (SUCCEEDED(hr)) IUnknown_Release(surface1);
+
     IDXGISurface_Release(surface);
     refcount = IDXGIDevice_Release(device);
     ok(!refcount, "Device has %u references left.\n", refcount);
@@ -478,19 +484,13 @@ static void test_createswapchain(void)
         hr = IDXGISwapChain_GetDesc(swapchain, &result_desc);
         ok(SUCCEEDED(hr), "GetDesc failed, hr %#x.\n", hr);
 
-        if (refresh_list[i].numerator_should_pass)
+        todo_wine_if (!refresh_list[i].numerator_should_pass)
             ok(result_desc.BufferDesc.RefreshRate.Numerator == refresh_list[i].numerator,
                 "Numerator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Numerator);
-        else
-            todo_wine ok(result_desc.BufferDesc.RefreshRate.Numerator == refresh_list[i].numerator,
-                "Numerator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Numerator);
 
-        if (refresh_list[i].denominator_should_pass)
+        todo_wine_if (!refresh_list[i].denominator_should_pass)
             ok(result_desc.BufferDesc.RefreshRate.Denominator == refresh_list[i].denominator,
                     "Denominator %u is %u.\n", i ,result_desc.BufferDesc.RefreshRate.Denominator);
-        else
-            todo_wine ok(result_desc.BufferDesc.RefreshRate.Denominator == refresh_list[i].denominator,
-                    "Denominator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Denominator);
 
         IDXGISwapChain_Release(swapchain);
     }
@@ -508,19 +508,13 @@ static void test_createswapchain(void)
         hr = IDXGISwapChain_GetDesc(swapchain, &result_desc);
         ok(SUCCEEDED(hr), "GetDesc failed, hr %#x.\n", hr);
 
-        if (refresh_list[i].numerator_should_pass)
+        todo_wine_if (!refresh_list[i].numerator_should_pass)
             ok(result_desc.BufferDesc.RefreshRate.Numerator == refresh_list[i].numerator,
                     "Numerator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Numerator);
-        else
-            todo_wine ok(result_desc.BufferDesc.RefreshRate.Numerator == refresh_list[i].numerator,
-                    "Numerator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Numerator);
 
-        if (refresh_list[i].denominator_should_pass)
+        todo_wine_if (!refresh_list[i].denominator_should_pass)
             ok(result_desc.BufferDesc.RefreshRate.Denominator == refresh_list[i].denominator,
                     "Denominator %u is %u.\n", i ,result_desc.BufferDesc.RefreshRate.Denominator);
-        else
-            todo_wine ok(result_desc.BufferDesc.RefreshRate.Denominator == refresh_list[i].denominator,
-                    "Denominator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Denominator);
 
         hr = IDXGISwapChain_SetFullscreenState(swapchain, FALSE, NULL);
         todo_wine ok(SUCCEEDED(hr), "SetFullscreenState failed, hr %#x.\n", hr);
