@@ -112,10 +112,11 @@ static const struct IClassFactoryVtbl hnetcfg_cf_vtbl =
 
 static hnetcfg_cf fw_manager_cf = { { &hnetcfg_cf_vtbl }, NetFwMgr_create };
 static hnetcfg_cf fw_app_cf = { { &hnetcfg_cf_vtbl }, NetFwAuthorizedApplication_create };
+static hnetcfg_cf fw_openport_cf = { { &hnetcfg_cf_vtbl }, NetFwOpenPort_create };
 
-BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID lpvReserved)
+BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID reserved)
 {
-    TRACE("(0x%p, %d, %p)\n",hInstDLL,fdwReason,lpvReserved);
+    TRACE("(0x%p, %d, %p)\n", hInstDLL, fdwReason, reserved);
 
     switch(fdwReason) {
         case DLL_WINE_PREATTACH:
@@ -124,6 +125,10 @@ BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID lpvReserved)
             instance = hInstDLL;
             DisableThreadLibraryCalls(hInstDLL);
             break;
+        case DLL_PROCESS_DETACH:
+            if (reserved) break;
+            release_typelib();
+            break;
     }
     return TRUE;
 }
@@ -142,6 +147,10 @@ HRESULT WINAPI DllGetClassObject( REFCLSID rclsid, REFIID iid, LPVOID *ppv )
     {
        cf = &fw_app_cf.IClassFactory_iface;
     }
+    else if (IsEqualGUID( rclsid, &CLSID_NetFwOpenPort ))
+    {
+       cf = &fw_openport_cf.IClassFactory_iface;
+    }
 
     if (!cf) return CLASS_E_CLASSNOTAVAILABLE;
     return IClassFactory_QueryInterface( cf, iid, ppv );
