@@ -34,7 +34,6 @@ typedef struct {
     bytecode_t *code;
     function_code_t *func_code;
     DWORD length;
-    jsdisp_t *arguments;
 } FunctionInstance;
 
 typedef struct {
@@ -43,9 +42,14 @@ typedef struct {
     jsdisp_t *var_obj;
 } ArgumentsInstance;
 
+static inline FunctionInstance *function_from_jsdisp(jsdisp_t *jsdisp)
+{
+    return CONTAINING_RECORD(jsdisp, FunctionInstance, dispex);
+}
+
 static inline FunctionInstance *function_from_vdisp(vdisp_t *vdisp)
 {
-    return (FunctionInstance*)vdisp->u.jsdisp;
+    return function_from_jsdisp(vdisp->u.jsdisp);
 }
 
 static inline FunctionInstance *function_this(vdisp_t *jsthis)
@@ -198,13 +202,17 @@ static HRESULT create_var_disp(script_ctx_t *ctx, FunctionInstance *function, un
 }
 
 static HRESULT invoke_source(script_ctx_t *ctx, FunctionInstance *function, IDispatch *this_obj, unsigned argc, jsval_t *argv,
-        jsval_t *r)
+        BOOL is_constructor, BOOL caller_execs_source, jsval_t *r)
 {
     jsdisp_t *var_disp, *arg_disp;
-    exec_ctx_t *exec_ctx;
     scope_chain_t *scope;
     HRESULT hres;
 
+    if(ctx->state == SCRIPTSTATE_UNINITIALIZED || ctx->state == SCRIPTSTATE_CLOSED) {
+        WARN("Script engine state does not allow running code.\n");
+        return E_UNEXPECTED;
+    }
+
     if(!function->func_code) {
         FIXME("no source\n");
         return E_FAIL;
@@ -229,57 +237,23 @@ static HRESULT invoke_source(script_ctx_t *ctx, FunctionInstance *function, IDis
 
     hres = scope_push(function->scope_chain, var_disp, to_disp(var_disp), &scope);
     if(SUCCEEDED(hres)) {
-        hres = create_exec_ctx(ctx, this_obj, var_disp, scope, FALSE, &exec_ctx);
+        DWORD exec_flags = 0;
+
+        if(caller_execs_source)
+            exec_flags |= EXEC_RETURN_TO_INTERP;
+        if(is_constructor)
+            exec_flags |= EXEC_CONSTRUCTOR;
+        hres = exec_source(ctx, exec_flags, function->code, function->func_code, scope, this_obj,
+                &function->dispex, var_disp, arg_disp, r);
+
         scope_release(scope);
-
-        if(SUCCEEDED(hres)) {
-            jsdisp_t *prev_args;
-
-            prev_args = function->arguments;
-            function->arguments = arg_disp;
-            hres = exec_source(exec_ctx, function->code, function->func_code, FALSE, r);
-            function->arguments = prev_args;
-
-            exec_release(exec_ctx);
-        }
     }
 
-    /* Reset arguments value to cut the reference cycle. Note that since all activation contexts have
-     * their own arguments property, it's impossible to use prototype's one during name lookup */
-    jsdisp_propput_name(var_disp, argumentsW, jsval_undefined());
-
     jsdisp_release(arg_disp);
     jsdisp_release(var_disp);
     return hres;
 }
 
-static HRESULT invoke_constructor(script_ctx_t *ctx, FunctionInstance *function, unsigned argc, jsval_t *argv,
-        jsval_t *r)
-{
-    jsdisp_t *this_obj;
-    jsval_t var;
-    HRESULT hres;
-
-    hres = create_object(ctx, &function->dispex, &this_obj);
-    if(FAILED(hres))
-        return hres;
-
-    hres = invoke_source(ctx, function, to_disp(this_obj), argc, argv, &var);
-    if(FAILED(hres)) {
-        jsdisp_release(this_obj);
-        return hres;
-    }
-
-    if(is_object_instance(var)) {
-        jsdisp_release(this_obj);
-        *r = var;
-    }else {
-        jsval_release(var);
-        *r = jsval_obj(this_obj);
-    }
-    return S_OK;
-}
-
 static HRESULT invoke_value_proc(script_ctx_t *ctx, FunctionInstance *function, IDispatch *this_disp, WORD flags,
         unsigned argc, jsval_t *argv, jsval_t *r)
 {
@@ -300,12 +274,12 @@ static HRESULT invoke_value_proc(script_ctx_t *ctx, FunctionInstance *function,
 }
 
 static HRESULT call_function(script_ctx_t *ctx, FunctionInstance *function, IDispatch *this_obj,
-        unsigned argc, jsval_t *argv, jsval_t *r)
+        unsigned argc, jsval_t *argv, BOOL caller_execs_source, jsval_t *r)
 {
     if(function->value_proc)
         return invoke_value_proc(ctx, function, this_obj, DISPATCH_METHOD, argc, argv, r);
 
-    return invoke_source(ctx, function, this_obj, argc, argv, r);
+    return invoke_source(ctx, function, this_obj, argc, argv, FALSE, caller_execs_source, r);
 }
 
 static HRESULT function_to_string(FunctionInstance *function, jsstr_t **ret)
@@ -340,6 +314,7 @@ static HRESULT function_to_string(FunctionInstance *function, jsstr_t **ret)
 
 HRESULT Function_invoke(jsdisp_t *func_this, IDispatch *jsthis, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
 {
+    const BOOL caller_execs_source = (flags & DISPATCH_JSCRIPT_CALLEREXECSSOURCE) != 0;
     FunctionInstance *function;
 
     TRACE("func %p this %p\n", func_this, jsthis);
@@ -347,35 +322,41 @@ HRESULT Function_invoke(jsdisp_t *func_this, IDispatch *jsthis, WORD flags, unsi
     assert(is_class(func_this, JSCLASS_FUNCTION));
     function = (FunctionInstance*)func_this;
 
+    flags &= ~DISPATCH_JSCRIPT_INTERNAL_MASK;
     if(function->value_proc)
         return invoke_value_proc(function->dispex.ctx, function, jsthis, flags, argc, argv, r);
 
-    if(flags == DISPATCH_CONSTRUCT)
-        return invoke_constructor(function->dispex.ctx, function, argc, argv, r);
+    if(flags == DISPATCH_CONSTRUCT) {
+        jsdisp_t *this_obj;
+        HRESULT hres;
 
-    assert(flags == DISPATCH_METHOD);
-    return invoke_source(function->dispex.ctx, function, jsthis, argc, argv, r);
-}
+        hres = create_object(function->dispex.ctx, &function->dispex, &this_obj);
+        if(FAILED(hres))
+            return hres;
 
-static HRESULT Function_length(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
-        jsval_t *r)
-{
-    FunctionInstance *This = function_from_vdisp(jsthis);
-
-    TRACE("%p %d\n", This, This->length);
-
-    switch(flags) {
-    case DISPATCH_PROPERTYGET:
-        *r = jsval_number(This->length);
-        break;
-    default:
-        FIXME("unimplemented flags %x\n", flags);
-        return E_NOTIMPL;
+        hres = invoke_source(function->dispex.ctx, function, to_disp(this_obj), argc, argv, TRUE, caller_execs_source, r);
+        jsdisp_release(this_obj);
+        return hres;
     }
 
+    assert(flags == DISPATCH_METHOD);
+    return invoke_source(function->dispex.ctx, function, jsthis, argc, argv, FALSE, caller_execs_source, r);
+}
+
+static HRESULT Function_get_length(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
+{
+    TRACE("%p\n", jsthis);
+
+    *r = jsval_number(function_from_jsdisp(jsthis)->length);
     return S_OK;
 }
 
+static HRESULT Function_set_length(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t value)
+{
+    FIXME("\n");
+    return E_NOTIMPL;
+}
+
 static HRESULT Function_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
@@ -478,7 +459,7 @@ static HRESULT Function_apply(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
     }
 
     if(SUCCEEDED(hres))
-        hres = call_function(ctx, function, this_obj, cnt, args, r);
+        hres = call_function(ctx, function, this_obj, cnt, args, (flags & DISPATCH_JSCRIPT_CALLEREXECSSOURCE) != 0, r);
 
     if(this_obj)
         IDispatch_Release(this_obj);
@@ -511,7 +492,7 @@ static HRESULT Function_call(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
         cnt = argc-1;
     }
 
-    hres = call_function(ctx, function, this_obj, cnt, argv+1, r);
+    hres = call_function(ctx, function, this_obj, cnt, argv+1, (flags & DISPATCH_JSCRIPT_CALLEREXECSSOURCE) != 0, r);
 
     if(this_obj)
         IDispatch_Release(this_obj);
@@ -532,56 +513,41 @@ HRESULT Function_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned
 
     function = (FunctionInstance*)jsthis->u.jsdisp;
 
-    switch(flags) {
-    case DISPATCH_METHOD:
-        assert(function->value_proc != NULL);
-        return invoke_value_proc(ctx, function, NULL, flags, argc, argv, r);
-
-    case DISPATCH_PROPERTYGET: {
-        HRESULT hres;
-        jsstr_t *str;
-
-        hres = function_to_string(function, &str);
-        if(FAILED(hres))
-            return hres;
-
-        *r = jsval_string(str);
-        break;
-    }
-
-    case DISPATCH_CONSTRUCT:
-        assert(function->value_proc != NULL);
-        return invoke_value_proc(ctx, function, NULL, flags, argc, argv, r);
-
-    default:
-        FIXME("not implemented flags %x\n", flags);
-        return E_NOTIMPL;
-    }
-
-    return S_OK;
+    assert(function->value_proc != NULL);
+    return invoke_value_proc(ctx, function, NULL, flags, argc, argv, r);
 }
 
-static HRESULT Function_arguments(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
-        unsigned argc, jsval_t *argv, jsval_t *r)
+HRESULT Function_get_value(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
 {
-    FunctionInstance *function = (FunctionInstance*)jsthis->u.jsdisp;
-    HRESULT hres = S_OK;
+    jsstr_t *str;
+    HRESULT hres;
 
     TRACE("\n");
 
-    switch(flags) {
-    case DISPATCH_PROPERTYGET: {
-        *r = function->arguments ? jsval_obj(jsdisp_addref(function->arguments)) : jsval_null();
-        break;
-    }
-    case DISPATCH_PROPERTYPUT:
-        break;
-    default:
-        FIXME("unimplemented flags %x\n", flags);
-        hres = E_NOTIMPL;
+    hres = function_to_string(function_from_jsdisp(jsthis), &str);
+    if(FAILED(hres))
+        return hres;
+
+    *r = jsval_string(str);
+    return S_OK;
+}
+
+static HRESULT Function_get_arguments(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
+{
+    FunctionInstance *function = function_from_jsdisp(jsthis);
+    call_frame_t *frame;
+
+    TRACE("\n");
+
+    for(frame = ctx->call_ctx; frame; frame = frame->prev_frame) {
+        if(frame->function_instance == &function->dispex) {
+            *r = jsval_obj(jsdisp_addref(frame->arguments_obj));
+            return S_OK;
+        }
     }
 
-    return hres;
+    *r = jsval_null();
+    return S_OK;
 }
 
 static void Function_destructor(jsdisp_t *dispex)
@@ -597,15 +563,15 @@ static void Function_destructor(jsdisp_t *dispex)
 
 static const builtin_prop_t Function_props[] = {
     {applyW,                 Function_apply,                 PROPF_METHOD|2},
-    {argumentsW,             Function_arguments,             0},
+    {argumentsW,             NULL, 0,                        Function_get_arguments, builtin_set_const},
     {callW,                  Function_call,                  PROPF_METHOD|1},
-    {lengthW,                Function_length,                0},
+    {lengthW,                NULL, 0,                        Function_get_length,    Function_set_length},
     {toStringW,              Function_toString,              PROPF_METHOD}
 };
 
 static const builtin_info_t Function_info = {
     JSCLASS_FUNCTION,
-    {NULL, Function_value, 0},
+    DEFAULT_FUNCTION_VALUE,
     sizeof(Function_props)/sizeof(*Function_props),
     Function_props,
     Function_destructor,
@@ -613,13 +579,13 @@ static const builtin_info_t Function_info = {
 };
 
 static const builtin_prop_t FunctionInst_props[] = {
-    {argumentsW,             Function_arguments,             0},
-    {lengthW,                Function_length,                0}
+    {argumentsW,             NULL, 0,                        Function_get_arguments, builtin_set_const},
+    {lengthW,                NULL, 0,                        Function_get_length,    Function_set_length}
 };
 
 static const builtin_info_t FunctionInst_info = {
     JSCLASS_FUNCTION,
-    {NULL, Function_value, 0},
+    DEFAULT_FUNCTION_VALUE,
     sizeof(FunctionInst_props)/sizeof(*FunctionInst_props),
     FunctionInst_props,
     Function_destructor,
@@ -839,6 +805,7 @@ static HRESULT FunctionConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD fla
     TRACE("\n");
 
     switch(flags) {
+    case DISPATCH_METHOD:
     case DISPATCH_CONSTRUCT: {
         IDispatch *ret;
 
