@@ -194,6 +194,24 @@ ok(testRecFunc.arguments === null, "testRecFunc.arguments = " + testRecFunc.argu
 testRecFunc(true);
 ok(testRecFunc.arguments === null, "testRecFunc.arguments = " + testRecFunc.arguments);
 
+function argumentsTest() {
+    var save = arguments;
+    with({arguments: 1}) {
+        ok(arguments === 1, "arguments = " + arguments);
+        (function() {
+            ok(argumentsTest.arguments === save, "unexpected argumentsTest.arguments");
+        })();
+    }
+    eval('ok(arguments === save, "unexpected arguments");');
+    [1,2].sort(function() {
+        ok(argumentsTest.arguments === save, "unexpected argumentsTest.arguments");
+        return 1;
+    });
+}
+/* FIXME: It seems that when function is called as an expression, instance object arguments is not set.
+ * We currently always set it in Wine. */
+argumentsTest();
+
 tmp = (function() {1;})();
 ok(tmp === undefined, "tmp = " + tmp);
 tmp = eval("1;");
@@ -1450,6 +1468,24 @@ function returnTest() {
 
 ok(returnTest() === undefined, "returnTest = " + returnTest());
 
+ActiveXObject = 1;
+ok(ActiveXObject === 1, "ActiveXObject = " + ActiveXObject);
+
+Boolean = 1;
+ok(Boolean === 1, "Boolean = " + Boolean);
+
+Object = 1;
+ok(Object === 1, "Object = " + Object);
+
+Array = 1;
+ok(Array === 1, "Array = " + Array);
+
+Date = 1;
+ok(Date === 1, "Date = " + Date);
+
+Error = 1;
+ok(Error === 1, "Error = " + Error);
+
 /* Keep this test in the end of file */
 undefined = 6;
 ok(undefined === 6, "undefined = " + undefined);
