@@ -887,6 +887,26 @@ BOOL WINAPI GetThreadIOPendingFlag( HANDLE thread, PBOOL io_pending )
     return TRUE;
 }
 
+/***********************************************************************
+ *              SetThreadPreferredUILanguages (KERNEL32.@)
+ */
+BOOL WINAPI SetThreadPreferredUILanguages( DWORD flags, PCZZWSTR buffer, PULONG count )
+{
+    FIXME( "%u, %p, %p\n", flags, buffer, count );
+    return TRUE;
+}
+
+/***********************************************************************
+ *              GetThreadPreferredUILanguages (KERNEL32.@)
+ */
+BOOL WINAPI GetThreadPreferredUILanguages( DWORD flags, PULONG count, PCZZWSTR buffer, PULONG buffersize )
+{
+    FIXME( "%u, %p, %p %p\n", flags, count, buffer, buffersize );
+    *count = 0;
+    *buffersize = 0;
+    return TRUE;
+}
+
 /***********************************************************************
  *              CallbackMayRunLong (KERNEL32.@)
  */
