@@ -73,7 +73,6 @@ static INT  test_OnPushContext = SINK_UNEXPECTED;
 static INT  test_OnPopContext = SINK_UNEXPECTED;
 static INT  test_KEV_OnSetFocus = SINK_UNEXPECTED;
 static INT  test_ACP_AdviseSink = SINK_UNEXPECTED;
-static INT  test_ACP_UnadviseSink = SINK_UNEXPECTED;
 static INT  test_ACP_GetStatus = SINK_UNEXPECTED;
 static INT  test_ACP_RequestLock = SINK_UNEXPECTED;
 static INT  test_ACP_GetEndACP = SINK_UNEXPECTED;
@@ -118,9 +117,7 @@ static inline void _sink_fire_ok(INT *sink, const CHAR* name)
             *sink = (*sink & ~SINK_EXPECTED_COUNT_MASK) + (count << 16);
             return;
         default:
-            if (todo)
-                todo_wine winetest_ok(0, "Unexpected %s sink\n",name);
-            else
+            todo_wine_if (todo)
                 winetest_ok(0, "Unexpected %s sink\n",name);
     }
     *sink = SINK_FIRED;
@@ -148,9 +145,7 @@ static inline void _sink_check_ok(INT *sink, const CHAR* name)
                 winetest_trace("optional sink %s not fired\n",name);
             break;
         default:
-            if (todo)
-                todo_wine winetest_ok(0, "%s not fired as expected, in state %x\n",name,*sink);
-            else
+            todo_wine_if (todo)
                 winetest_ok(0, "%s not fired as expected, in state %x\n",name,*sink);
     }
     *sink = SINK_UNEXPECTED;
@@ -236,9 +231,6 @@ static HRESULT WINAPI TextStoreACP_AdviseSink(ITextStoreACP *iface,
 
     sink_fire_ok(&test_ACP_AdviseSink,"TextStoreACP_AdviseSink");
 
-    if(ACPSink)
-        return S_OK;
-
     hr = IUnknown_QueryInterface(punk, &IID_ITextStoreACPSink, (void**)&ACPSink);
     ok(SUCCEEDED(hr),"Unable to QueryInterface on sink\n");
 
@@ -252,7 +244,7 @@ static HRESULT WINAPI TextStoreACP_AdviseSink(ITextStoreACP *iface,
 static HRESULT WINAPI TextStoreACP_UnadviseSink(ITextStoreACP *iface,
     IUnknown *punk)
 {
-    sink_fire_ok(&test_ACP_UnadviseSink,"TextStoreACP_UnadviseSink");
+    trace("\n");
     return S_OK;
 }
 
@@ -1417,7 +1409,7 @@ static void test_startSession(void)
     DWORD editCookie;
     ITfDocumentMgr *dmtest;
     ITfContext *cxt,*cxt2,*cxt3,*cxtTest;
-    ITextStoreACP *ts = NULL;
+    ITextStoreACP *ts;
     TfClientId cid2 = 0;
     ITfThreadMgrEx *tmex;
 
@@ -1435,13 +1427,16 @@ static void test_startSession(void)
     ok(cid == cid2, "Second activate client ID does not match\n");
 
     hr = ITfThreadMgr_QueryInterface(g_tm, &IID_ITfThreadMgrEx, (void **)&tmex);
-    ok(SUCCEEDED(hr), "Unable to acquire ITfThreadMgrEx interface\n");
+    if (hr == S_OK)
+    {
+        hr = ITfThreadMgrEx_ActivateEx(tmex, &cid2, 0);
+        ok(SUCCEEDED(hr), "Failed to Activate\n");
+        ok(cid == cid2, "ActivateEx client ID does not match\n");
 
-    hr = ITfThreadMgrEx_ActivateEx(tmex, &cid2, 0);
-    ok(SUCCEEDED(hr), "Failed to Activate\n");
-    ok(cid == cid2, "ActivateEx client ID does not match\n");
-
-    ITfThreadMgrEx_Release(tmex);
+        ITfThreadMgrEx_Release(tmex);
+    }
+    else
+        win_skip("ITfThreadMgrEx is not supported\n");
 
     hr = ITfThreadMgr_Deactivate(g_tm);
     ok(SUCCEEDED(hr), "Failed to Deactivate\n");
@@ -1581,8 +1576,6 @@ static void test_startSession(void)
     ok(hr == S_FALSE, "ITfContext_GetDocumentMgr wrong rc %x\n",hr);
     ok(dmtest == NULL,"returned documentmgr should be null\n");
 
-    ITfContext_Release(cxt2);
-
     hr = ITfDocumentMgr_GetTop(g_dm, &cxtTest);
     ok(SUCCEEDED(hr),"GetTop Failed\n");
     ok(cxtTest == cxt, "Wrong context on top\n");
@@ -1593,25 +1586,6 @@ static void test_startSession(void)
     ok(cxtTest == cxt, "Wrong context on base\n");
     ITfContext_Release(cxtTest);
 
-    hr = ITfDocumentMgr_CreateContext(g_dm, cid, 0, (IUnknown*)ts, &cxt2, &editCookie);
-    ok(hr == S_OK,"CreateContext Failed\n");
-
-    test_OnPushContext = SINK_EXPECTED;
-    test_ACP_AdviseSink = SINK_EXPECTED;
-    hr = ITfDocumentMgr_Push(g_dm, cxt2);
-    ok(hr == S_OK,"Push Failed\n");
-    sink_check_ok(&test_OnPushContext,"OnPushContext");
-    sink_check_ok(&test_ACP_AdviseSink,"TextStoreACP_AdviseSink");
-
-    test_ACP_UnadviseSink = SINK_EXPECTED;
-    cnt = check_context_refcount(cxt2);
-    test_OnPopContext = SINK_EXPECTED;
-    hr = ITfDocumentMgr_Pop(g_dm, 0);
-    ok(hr == S_OK,"Pop Failed\n");
-    ok(check_context_refcount(cxt2) < cnt, "Ref count did not decrease\n");
-    sink_check_ok(&test_OnPopContext,"OnPopContext");
-    sink_check_ok(&test_ACP_UnadviseSink,"TextStoreACP_AdviseSink");
-
     hr = ITfDocumentMgr_Pop(g_dm, 0);
     ok(FAILED(hr),"Pop Succeeded\n");
 
