@@ -57,6 +57,8 @@ typedef enum
     WIN2K8R2,/* Windows 2008 R2 */
     WIN7,    /* Windows 7 */
     WIN8,    /* Windows 8 */
+    WIN81,   /* Windows 8.1 */
+    WIN10,   /* Windows 10 */
     NB_WINDOWS_VERSIONS
 } WINDOWS_VERSION;
 
@@ -138,8 +140,8 @@ static const RTL_OSVERSIONINFOEXW VersionData[NB_WINDOWS_VERSIONS] =
     /* WINXP64 */
     {
         sizeof(RTL_OSVERSIONINFOEXW), 5, 2, 0xECE, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','1',0},
-        1, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','2',0},
+        2, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
     },
     /* WIN2K3 */
     {
@@ -177,6 +179,18 @@ static const RTL_OSVERSIONINFOEXW VersionData[NB_WINDOWS_VERSIONS] =
         {' ',0},
         0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
     },
+    /* WIN81 */
+    {
+        sizeof(RTL_OSVERSIONINFOEXW), 6, 3, 0x2580, VER_PLATFORM_WIN32_NT,
+        {' ',0},
+        0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+    },
+    /* WIN10 */
+    {
+        sizeof(RTL_OSVERSIONINFOEXW), 10, 0, 0x2800, VER_PLATFORM_WIN32_NT,
+        {' ',0},
+        0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+    },
 
 };
 
@@ -199,6 +213,8 @@ static const char * const WinVersionNames[NB_WINDOWS_VERSIONS] =
     "win2008r2,win2k8r2",         /* WIN2K8R2 */
     "win7",                       /* WIN7 */
     "win8",                       /* WIN8 */
+    "win81",                      /* WIN81 */
+    "win10",                      /* WIN10 */
 };
 
 
