@@ -472,7 +472,7 @@ static HRESULT WINAPI OLEPictureImpl_QueryInterface(
 
   if (!*ppvObject)
   {
-    FIXME("() : asking for un supported interface %s\n",debugstr_guid(riid));
+    FIXME("() : asking for unsupported interface %s\n",debugstr_guid(riid));
     return E_NOINTERFACE;
   }
 
@@ -2315,6 +2315,15 @@ HRESULT WINAPI OleLoadPictureEx( LPSTREAM lpstream, LONG lSize, BOOL fRunmode,
   return hr;
 }
 
+/***********************************************************************
+ * OleLoadPictureFile (OLEAUT32.422)
+ */
+HRESULT WINAPI OleLoadPictureFile(VARIANT file, LPDISPATCH *picture)
+{
+    FIXME("(%s %p): stub\n", wine_dbgstr_variant(&file), picture);
+    return E_NOTIMPL;
+}
+
 /***********************************************************************
  * OleSavePictureFile (OLEAUT32.423)
  */
