@@ -18,7 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 #include <stdarg.h>
-#include <assert.h>
 
 #define COBJMACROS
 #include "windef.h"
@@ -78,6 +77,19 @@ static inline IQueryAssociationsImpl *impl_from_IQueryAssociations(IQueryAssocia
   return CONTAINING_RECORD(iface, IQueryAssociationsImpl, IQueryAssociations_iface);
 }
 
+struct enumassochandlers
+{
+    IEnumAssocHandlers IEnumAssocHandlers_iface;
+    LONG ref;
+};
+
+static inline struct enumassochandlers *impl_from_IEnumAssocHandlers(IEnumAssocHandlers *iface)
+{
+  return CONTAINING_RECORD(iface, struct enumassochandlers, IEnumAssocHandlers_iface);
+}
+
+static HRESULT ASSOC_GetValue(HKEY hkey, const WCHAR *name, void **data, DWORD *data_size);
+
 /**************************************************************************
  *  IQueryAssociations_QueryInterface
  *
@@ -100,7 +112,7 @@ static HRESULT WINAPI IQueryAssociations_fnQueryInterface(
   if (IsEqualIID(riid, &IID_IUnknown) ||
       IsEqualIID(riid, &IID_IQueryAssociations))
   {
-    *ppvObj = This;
+    *ppvObj = &This->IQueryAssociations_iface;
 
     IQueryAssociations_AddRef((IQueryAssociations*)*ppvObj);
     TRACE("Returning IQueryAssociations (%p)\n", *ppvObj);
@@ -184,31 +196,70 @@ static HRESULT WINAPI IQueryAssociations_fnInit(
         FIXME("hwnd != NULL not supported\n");
     if (cfFlags != 0)
 	FIXME("unsupported flags: %x\n", cfFlags);
+
+    RegCloseKey(This->hkeySource);
+    if (This->hkeySource != This->hkeyProgID)
+        RegCloseKey(This->hkeyProgID);
+    This->hkeySource = This->hkeyProgID = NULL;
+
+    /* If the process of initializing hkeyProgID fails, just return S_OK. That's what Windows does. */
     if (pszAssoc != NULL)
     {
+        WCHAR *progId;
+        HRESULT hr;
+
         ret = RegOpenKeyExW(HKEY_CLASSES_ROOT,
                             pszAssoc,
                             0,
                             KEY_READ,
                             &This->hkeySource);
-        if (ret != ERROR_SUCCESS)
-            return E_FAIL;
-        /* if this is not a prog id */
-        if ((*pszAssoc == '.') || (*pszAssoc == '{'))
+        if (ret)
+            return S_OK;
+        /* if this is a progid */
+        if (*pszAssoc != '.' && *pszAssoc != '{')
         {
-            RegOpenKeyExW(This->hkeySource,
-                          szProgID,
-                          0,
-                          KEY_READ,
-                          &This->hkeyProgID);
-        }
-        else
             This->hkeyProgID = This->hkeySource;
+            return S_OK;
+        }
+
+        /* if it's not a progid, it's a file extension or clsid */
+        if (*pszAssoc == '.')
+        {
+            /* for a file extension, the progid is the default value */
+            hr = ASSOC_GetValue(This->hkeySource, NULL, (void**)&progId, NULL);
+            if (FAILED(hr))
+                return S_OK;
+        }
+        else /* if (*pszAssoc == '{') */
+        {
+            HKEY progIdKey;
+            /* for a clsid, the progid is the default value of the ProgID subkey */
+            ret = RegOpenKeyExW(This->hkeySource,
+                                szProgID,
+                                0,
+                                KEY_READ,
+                                &progIdKey);
+            if (ret != ERROR_SUCCESS)
+                return S_OK;
+            hr = ASSOC_GetValue(progIdKey, NULL, (void**)&progId, NULL);
+            if (FAILED(hr))
+                return S_OK;
+            RegCloseKey(progIdKey);
+        }
+
+        /* open the actual progid key, the one with the shell subkey */
+        ret = RegOpenKeyExW(HKEY_CLASSES_ROOT,
+                            progId,
+                            0,
+                            KEY_READ,
+                            &This->hkeyProgID);
+        HeapFree(GetProcessHeap(), 0, progId);
+
         return S_OK;
     }
     else if (hkeyProgid != NULL)
     {
-        This->hkeyProgID = hkeyProgid;
+        This->hkeySource = This->hkeyProgID = hkeyProgid;
         return S_OK;
     }
     else
@@ -220,7 +271,6 @@ static HRESULT ASSOC_GetValue(HKEY hkey, const WCHAR *name, void **data, DWORD *
   DWORD size;
   LONG ret;
 
-  assert(data);
   ret = RegQueryValueExW(hkey, name, 0, NULL, NULL, &size);
   if (ret != ERROR_SUCCESS)
     return HRESULT_FROM_WIN32(ret);
@@ -240,79 +290,88 @@ static HRESULT ASSOC_GetValue(HKEY hkey, const WCHAR *name, void **data, DWORD *
   return S_OK;
 }
 
-static HRESULT ASSOC_GetCommand(IQueryAssociationsImpl *This,
-                                LPCWSTR pszExtra, WCHAR **ppszCommand)
+static HRESULT ASSOC_GetCommand(IQueryAssociationsImpl *This, const WCHAR *extra, WCHAR **command)
 {
   HKEY hkeyCommand;
-  HKEY hkeyFile;
   HKEY hkeyShell;
   HKEY hkeyVerb;
   HRESULT hr;
   LONG ret;
-  WCHAR * pszExtraFromReg = NULL;
-  WCHAR * pszFileType;
+  WCHAR *extra_from_reg = NULL;
+  WCHAR *filetype;
   static const WCHAR commandW[] = { 'c','o','m','m','a','n','d',0 };
   static const WCHAR shellW[] = { 's','h','e','l','l',0 };
 
-  hr = ASSOC_GetValue(This->hkeySource, NULL, (void**)&pszFileType, NULL);
-  if (FAILED(hr))
-    return hr;
-  ret = RegOpenKeyExW(HKEY_CLASSES_ROOT, pszFileType, 0, KEY_READ, &hkeyFile);
-  HeapFree(GetProcessHeap(), 0, pszFileType);
-  if (ret != ERROR_SUCCESS)
-    return HRESULT_FROM_WIN32(ret);
-
-  ret = RegOpenKeyExW(hkeyFile, shellW, 0, KEY_READ, &hkeyShell);
-  RegCloseKey(hkeyFile);
-  if (ret != ERROR_SUCCESS)
-    return HRESULT_FROM_WIN32(ret);
-
-  if (!pszExtra)
+  /* When looking for file extension it's possible to have a default value
+     that points to another key that contains 'shell/<verb>/command' subtree. */
+  hr = ASSOC_GetValue(This->hkeySource, NULL, (void**)&filetype, NULL);
+  if (hr == S_OK)
   {
-    hr = ASSOC_GetValue(hkeyShell, NULL, (void**)&pszExtraFromReg, NULL);
-    /* if no default action */
-    if (hr == E_FAIL || hr == HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND))
-    {
-      DWORD rlen;
-      ret = RegQueryInfoKeyW(hkeyShell, 0, 0, 0, 0, &rlen, 0, 0, 0, 0, 0, 0);
-      if (ret != ERROR_SUCCESS)
+      HKEY hkeyFile;
+
+      ret = RegOpenKeyExW(HKEY_CLASSES_ROOT, filetype, 0, KEY_READ, &hkeyFile);
+      HeapFree(GetProcessHeap(), 0, filetype);
+
+      if (ret == ERROR_SUCCESS)
       {
-        RegCloseKey(hkeyShell);
-        return HRESULT_FROM_WIN32(ret);
+          ret = RegOpenKeyExW(hkeyFile, shellW, 0, KEY_READ, &hkeyShell);
+          RegCloseKey(hkeyFile);
       }
-      rlen++;
-      pszExtraFromReg = HeapAlloc(GetProcessHeap(), 0, rlen * sizeof(WCHAR));
-      if (!pszExtraFromReg)
+      else
+          ret = RegOpenKeyExW(This->hkeySource, shellW, 0, KEY_READ, &hkeyShell);
+  }
+  else
+      ret = RegOpenKeyExW(This->hkeySource, shellW, 0, KEY_READ, &hkeyShell);
+
+  if (ret) return HRESULT_FROM_WIN32(ret);
+
+  if (!extra)
+  {
+      /* check for default verb */
+      hr = ASSOC_GetValue(hkeyShell, NULL, (void**)&extra_from_reg, NULL);
+      if (FAILED(hr))
       {
-        RegCloseKey(hkeyShell);
-        return E_OUTOFMEMORY;
+          /* no default verb, try first subkey */
+          DWORD max_subkey_len;
+
+          ret = RegQueryInfoKeyW(hkeyShell, NULL, NULL, NULL, NULL, &max_subkey_len, NULL, NULL, NULL, NULL, NULL, NULL);
+          if (ret)
+          {
+              RegCloseKey(hkeyShell);
+              return HRESULT_FROM_WIN32(ret);
+          }
+
+          max_subkey_len++;
+          extra_from_reg = HeapAlloc(GetProcessHeap(), 0, max_subkey_len * sizeof(WCHAR));
+          if (!extra_from_reg)
+          {
+              RegCloseKey(hkeyShell);
+              return E_OUTOFMEMORY;
+          }
+
+          ret = RegEnumKeyExW(hkeyShell, 0, extra_from_reg, &max_subkey_len, NULL, NULL, NULL, NULL);
+          if (ret)
+          {
+              HeapFree(GetProcessHeap(), 0, extra_from_reg);
+              RegCloseKey(hkeyShell);
+              return HRESULT_FROM_WIN32(ret);
+          }
       }
-      ret = RegEnumKeyExW(hkeyShell, 0, pszExtraFromReg, &rlen, 0, NULL, NULL, NULL);
-      if (ret != ERROR_SUCCESS)
-      {
-        RegCloseKey(hkeyShell);
-        return HRESULT_FROM_WIN32(ret);
-      }
-    }
-    else if (FAILED(hr))
-    {
-      RegCloseKey(hkeyShell);
-      return hr;
-    }
+      extra = extra_from_reg;
   }
 
-  ret = RegOpenKeyExW(hkeyShell, pszExtra ? pszExtra : pszExtraFromReg, 0,
-                      KEY_READ, &hkeyVerb);
-  HeapFree(GetProcessHeap(), 0, pszExtraFromReg);
+  /* open verb subkey */
+  ret = RegOpenKeyExW(hkeyShell, extra, 0, KEY_READ, &hkeyVerb);
+  HeapFree(GetProcessHeap(), 0, extra_from_reg);
   RegCloseKey(hkeyShell);
-  if (ret != ERROR_SUCCESS)
-    return HRESULT_FROM_WIN32(ret);
+  if (ret) return HRESULT_FROM_WIN32(ret);
 
+  /* open command subkey */
   ret = RegOpenKeyExW(hkeyVerb, commandW, 0, KEY_READ, &hkeyCommand);
   RegCloseKey(hkeyVerb);
-  if (ret != ERROR_SUCCESS)
-    return HRESULT_FROM_WIN32(ret);
-  hr = ASSOC_GetValue(hkeyCommand, NULL, (void**)ppszCommand, NULL);
+  if (ret) return HRESULT_FROM_WIN32(ret);
+
+  hr = ASSOC_GetValue(hkeyCommand, NULL, (void**)command, NULL);
   RegCloseKey(hkeyCommand);
   return hr;
 }
@@ -326,8 +385,6 @@ static HRESULT ASSOC_GetExecutable(IQueryAssociationsImpl *This,
   WCHAR *pszEnd;
   HRESULT hr;
 
-  assert(len);
-
   hr = ASSOC_GetCommand(This, pszExtra, &pszCommand);
   if (FAILED(hr))
     return hr;
@@ -365,8 +422,6 @@ static HRESULT ASSOC_GetExecutable(IQueryAssociationsImpl *This,
 static HRESULT ASSOC_ReturnData(void *out, DWORD *outlen, const void *data,
                                 DWORD datalen)
 {
-  assert(outlen);
-
   if (out)
   {
     if (*outlen < datalen)
@@ -385,17 +440,41 @@ static HRESULT ASSOC_ReturnData(void *out, DWORD *outlen, const void *data,
   }
 }
 
-static HRESULT ASSOC_ReturnString(LPWSTR out, DWORD *outlen, LPCWSTR data,
-        DWORD datalen)
+static HRESULT ASSOC_ReturnString(ASSOCF flags, LPWSTR out, DWORD *outlen, LPCWSTR data, DWORD datalen)
 {
-    HRESULT hres;
+    HRESULT hr = S_OK;
+    DWORD len;
 
-    assert(outlen);
+    TRACE("flags=0x%08x, data=%s\n", flags, debugstr_w(data));
 
-    *outlen *= sizeof(WCHAR);
-    hres = ASSOC_ReturnData(out, outlen, data, datalen*sizeof(WCHAR));
-    *outlen /= sizeof(WCHAR);
-    return hres;
+    if (!out)
+    {
+        *outlen = datalen;
+        return S_FALSE;
+    }
+
+    if (*outlen < datalen)
+    {
+        if (flags & ASSOCF_NOTRUNCATE)
+        {
+            len = 0;
+            if (*outlen > 0) out[0] = 0;
+            hr = E_POINTER;
+        }
+        else
+        {
+            len = min(*outlen, datalen);
+            hr = E_NOT_SUFFICIENT_BUFFER;
+        }
+        *outlen = datalen;
+    }
+    else
+        len = datalen;
+
+    if (len)
+        memcpy(out, data, len*sizeof(WCHAR));
+
+    return hr;
 }
 
 /**************************************************************************
@@ -417,27 +496,29 @@ static HRESULT ASSOC_ReturnString(LPWSTR out, DWORD *outlen, LPCWSTR data,
  */
 static HRESULT WINAPI IQueryAssociations_fnGetString(
   IQueryAssociations *iface,
-  ASSOCF cfFlags,
+  ASSOCF flags,
   ASSOCSTR str,
   LPCWSTR pszExtra,
   LPWSTR pszOut,
   DWORD *pcchOut)
 {
   IQueryAssociationsImpl *This = impl_from_IQueryAssociations(iface);
-  const ASSOCF cfUnimplemented = ~(0);
+  const ASSOCF unimplemented_flags = ~ASSOCF_NOTRUNCATE;
   DWORD len = 0;
   HRESULT hr;
   WCHAR path[MAX_PATH];
 
-  TRACE("(%p,0x%08x,%u,%s,%p,%p)\n", This, cfFlags, str,
-        debugstr_w(pszExtra), pszOut, pcchOut);
+  TRACE("(%p)->(0x%08x, %u, %s, %p, %p)\n", This, flags, str, debugstr_w(pszExtra), pszOut, pcchOut);
 
-  if (cfFlags & cfUnimplemented)
-    FIXME("%08x: unimplemented flags!\n", cfFlags & cfUnimplemented);
+  if (flags & unimplemented_flags)
+    FIXME("%08x: unimplemented flags\n", flags & unimplemented_flags);
 
   if (!pcchOut)
     return E_UNEXPECTED;
 
+  if (!This->hkeySource && !This->hkeyProgID)
+    return HRESULT_FROM_WIN32(ERROR_NO_ASSOCIATION);
+
   switch (str)
   {
     case ASSOCSTR_COMMAND:
@@ -446,7 +527,7 @@ static HRESULT WINAPI IQueryAssociations_fnGetString(
       hr = ASSOC_GetCommand(This, pszExtra, &command);
       if (SUCCEEDED(hr))
       {
-        hr = ASSOC_ReturnString(pszOut, pcchOut, command, strlenW(command) + 1);
+        hr = ASSOC_ReturnString(flags, pszOut, pcchOut, command, strlenW(command) + 1);
         HeapFree(GetProcessHeap(), 0, command);
       }
       return hr;
@@ -458,38 +539,20 @@ static HRESULT WINAPI IQueryAssociations_fnGetString(
       if (FAILED(hr))
         return hr;
       len++;
-      return ASSOC_ReturnString(pszOut, pcchOut, path, len);
+      return ASSOC_ReturnString(flags, pszOut, pcchOut, path, len);
     }
 
     case ASSOCSTR_FRIENDLYDOCNAME:
     {
-      WCHAR *pszFileType;
-      DWORD ret;
-      DWORD size;
+      WCHAR *docName;
 
-      hr = ASSOC_GetValue(This->hkeySource, NULL, (void**)&pszFileType, NULL);
-      if (FAILED(hr))
-        return hr;
-      size = 0;
-      ret = RegGetValueW(HKEY_CLASSES_ROOT, pszFileType, NULL, RRF_RT_REG_SZ, NULL, NULL, &size);
-      if (ret == ERROR_SUCCESS)
-      {
-        WCHAR *docName = HeapAlloc(GetProcessHeap(), 0, size);
-        if (docName)
-        {
-          ret = RegGetValueW(HKEY_CLASSES_ROOT, pszFileType, NULL, RRF_RT_REG_SZ, NULL, docName, &size);
-          if (ret == ERROR_SUCCESS)
-            hr = ASSOC_ReturnString(pszOut, pcchOut, docName, strlenW(docName) + 1);
-          else
-            hr = HRESULT_FROM_WIN32(ret);
-          HeapFree(GetProcessHeap(), 0, docName);
-        }
-        else
-          hr = E_OUTOFMEMORY;
+      hr = ASSOC_GetValue(This->hkeyProgID, NULL, (void**)&docName, NULL);
+      if (FAILED(hr)) {
+          /* hKeyProgID is NULL or there is no default value, so fail */
+          return HRESULT_FROM_WIN32(ERROR_NO_ASSOCIATION);
       }
-      else
-        hr = HRESULT_FROM_WIN32(ret);
-      HeapFree(GetProcessHeap(), 0, pszFileType);
+      hr = ASSOC_ReturnString(flags, pszOut, pcchOut, docName, strlenW(docName) + 1);
+      HeapFree(GetProcessHeap(), 0, docName);
       return hr;
     }
 
@@ -535,7 +598,7 @@ static HRESULT WINAPI IQueryAssociations_fnGetString(
             /* Does strlenW(bufW) == 0 mean we use the filename? */
             len = strlenW(bufW) + 1;
             TRACE("found FileDescription: %s\n", debugstr_w(bufW));
-            hr = ASSOC_ReturnString(pszOut, pcchOut, bufW, len);
+            hr = ASSOC_ReturnString(flags, pszOut, pcchOut, bufW, len);
             HeapFree(GetProcessHeap(), 0, verinfoW);
             return hr;
           }
@@ -545,7 +608,7 @@ get_friendly_name_fail:
       PathRemoveExtensionW(path);
       PathStripPathW(path);
       TRACE("using filename: %s\n", debugstr_w(path));
-      hr = ASSOC_ReturnString(pszOut, pcchOut, path, strlenW(path) + 1);
+      hr = ASSOC_ReturnString(flags, pszOut, pcchOut, path, strlenW(path) + 1);
       HeapFree(GetProcessHeap(), 0, verinfoW);
       return hr;
     }
@@ -566,7 +629,7 @@ get_friendly_name_fail:
       {
         ret = RegGetValueW(This->hkeySource, NULL, Content_TypeW, RRF_RT_REG_SZ, NULL, contentType, &size);
         if (ret == ERROR_SUCCESS)
-          hr = ASSOC_ReturnString(pszOut, pcchOut, contentType, strlenW(contentType) + 1);
+          hr = ASSOC_ReturnString(flags, pszOut, pcchOut, contentType, strlenW(contentType) + 1);
         else
           hr = HRESULT_FROM_WIN32(ret);
         HeapFree(GetProcessHeap(), 0, contentType);
@@ -579,41 +642,33 @@ get_friendly_name_fail:
     case ASSOCSTR_DEFAULTICON:
     {
       static const WCHAR DefaultIconW[] = {'D','e','f','a','u','l','t','I','c','o','n',0};
-      WCHAR *pszFileType;
+      static const WCHAR documentIcon[] = {'s','h','e','l','l','3','2','.','d','l','l',',','0',0};
       DWORD ret;
       DWORD size;
-      HKEY hkeyFile;
 
-      hr = ASSOC_GetValue(This->hkeySource, NULL, (void**)&pszFileType, NULL);
-      if (FAILED(hr))
-        return hr;
-      ret = RegOpenKeyExW(HKEY_CLASSES_ROOT, pszFileType, 0, KEY_READ, &hkeyFile);
+      size = 0;
+      ret = RegGetValueW(This->hkeyProgID, DefaultIconW, NULL, RRF_RT_REG_SZ, NULL, NULL, &size);
       if (ret == ERROR_SUCCESS)
       {
-        size = 0;
-        ret = RegGetValueW(hkeyFile, DefaultIconW, NULL, RRF_RT_REG_SZ, NULL, NULL, &size);
-        if (ret == ERROR_SUCCESS)
+        WCHAR *icon = HeapAlloc(GetProcessHeap(), 0, size);
+        if (icon)
         {
-          WCHAR *icon = HeapAlloc(GetProcessHeap(), 0, size);
-          if (icon)
-          {
-            ret = RegGetValueW(hkeyFile, DefaultIconW, NULL, RRF_RT_REG_SZ, NULL, icon, &size);
-            if (ret == ERROR_SUCCESS)
-              hr = ASSOC_ReturnString(pszOut, pcchOut, icon, strlenW(icon) + 1);
-            else
-              hr = HRESULT_FROM_WIN32(ret);
-            HeapFree(GetProcessHeap(), 0, icon);
-          }
+          ret = RegGetValueW(This->hkeyProgID, DefaultIconW, NULL, RRF_RT_REG_SZ, NULL, icon, &size);
+          if (ret == ERROR_SUCCESS)
+            hr = ASSOC_ReturnString(flags, pszOut, pcchOut, icon, strlenW(icon) + 1);
           else
-            hr = E_OUTOFMEMORY;
+            hr = HRESULT_FROM_WIN32(ret);
+          HeapFree(GetProcessHeap(), 0, icon);
         }
         else
-          hr = HRESULT_FROM_WIN32(ret);
-        RegCloseKey(hkeyFile);
+          hr = E_OUTOFMEMORY;
+      } else {
+          /* there is no DefaultIcon subkey or hkeyProgID is NULL, so return the default document icon */
+          if (This->hkeyProgID == NULL)
+              hr = ASSOC_ReturnString(flags, pszOut, pcchOut, documentIcon, strlenW(documentIcon) + 1);
+          else
+              return HRESULT_FROM_WIN32(ERROR_NO_ASSOCIATION);
       }
-      else
-        hr = HRESULT_FROM_WIN32(ret);
-      HeapFree(GetProcessHeap(), 0, pszFileType);
       return hr;
     }
     case ASSOCSTR_SHELLEXTENSION:
@@ -638,7 +693,7 @@ get_friendly_name_fail:
         RegCloseKey(hkey);
         if (ret) return HRESULT_FROM_WIN32(ret);
 
-        return ASSOC_ReturnString(pszOut, pcchOut, guid, size / sizeof(WCHAR));
+        return ASSOC_ReturnString(flags, pszOut, pcchOut, guid, size / sizeof(WCHAR));
     }
 
     default:
@@ -701,7 +756,7 @@ static HRESULT WINAPI IQueryAssociations_fnGetData(IQueryAssociations *iface,
     static const WCHAR edit_flags[] = {'E','d','i','t','F','l','a','g','s',0};
 
     IQueryAssociationsImpl *This = impl_from_IQueryAssociations(iface);
-    void *data;
+    void *data = NULL;
     DWORD size;
     HRESULT hres;
 
@@ -717,10 +772,8 @@ static HRESULT WINAPI IQueryAssociations_fnGetData(IQueryAssociations *iface,
             return HRESULT_FROM_WIN32(ERROR_NO_ASSOCIATION);
 
         hres = ASSOC_GetValue(This->hkeyProgID, edit_flags, &data, &size);
-        if(FAILED(hres) || !pcbOut)
-            return hres;
-
-        hres = ASSOC_ReturnData(pvOut, pcbOut, data, size);
+        if(SUCCEEDED(hres) && pcbOut)
+            hres = ASSOC_ReturnData(pvOut, pcbOut, data, size);
         HeapFree(GetProcessHeap(), 0, data);
         return hres;
     default:
@@ -829,11 +882,105 @@ static ULONG WINAPI ApplicationAssociationRegistration_Release(IApplicationAssoc
     return ref;
 }
 
-static HRESULT WINAPI ApplicationAssociationRegistration_QueryCurrentDefault(IApplicationAssociationRegistration* This, LPCWSTR query,
+static HRESULT WINAPI ApplicationAssociationRegistration_QueryCurrentDefault(IApplicationAssociationRegistration *iface, LPCWSTR query,
                                                                              ASSOCIATIONTYPE type, ASSOCIATIONLEVEL level, LPWSTR *association)
 {
-    FIXME("(%p)->(%s, %d, %d, %p)\n", This, debugstr_w(query), type, level, association);
-    return E_NOTIMPL;
+    IApplicationAssociationRegistrationImpl *This = impl_from_IApplicationAssociationRegistration(iface);
+    static WCHAR urlassoc[] = {'U','r','l','A','s','s','o','c','i','a','t','i','o','n','s',0};
+    static WCHAR mimeassoc[] = {'M','I','M','E','A','s','s','o','c','i','a','t','i','o','n','s',0};
+    static WCHAR assocations[] = {'S','o','f','t','w','a','r','e','\\','M','i','c','r','o','s','o','f','t','\\',
+                                'W','i','n','d','o','w','s','\\','S','h','e','l','l','\\',
+                                'A','s','s','o','c','i','a','t','i','o','n','s',0};
+    static WCHAR slash[] = {'\\',0};
+    static WCHAR choice[] = {'U','s','e','r','C','h','o','i','c','e',0};
+    static WCHAR propid[] = {'P','r','o','g','i','d',0};
+    WCHAR path[MAX_PATH] = {0};
+    DWORD ret, keytype, size;
+    HKEY hkey = NULL;
+    HRESULT hr = HRESULT_FROM_WIN32(ERROR_NO_ASSOCIATION);
+
+    TRACE("(%p)->(%s, %d, %d, %p)\n", This, debugstr_w(query), type, level, association);
+
+    if(!association)
+        return E_INVALIDARG;
+
+    *association = NULL;
+
+    if((type == AT_URLPROTOCOL || type == AT_FILEEXTENSION) && !query[0])
+        return E_INVALIDARG;
+    else if(type == AT_FILEEXTENSION && query[0] != '.')
+        return E_INVALIDARG;
+
+    if(type == AT_FILEEXTENSION)
+    {
+        ret = RegOpenKeyExW(HKEY_CLASSES_ROOT, query, 0, KEY_READ, &hkey);
+        if(ret == ERROR_SUCCESS)
+        {
+            ret = RegGetValueW(hkey, NULL, NULL, RRF_RT_REG_SZ, &keytype, NULL, &size);
+            if(ret == ERROR_SUCCESS)
+            {
+                *association = CoTaskMemAlloc(size);
+                if(*association)
+                {
+                    ret = RegGetValueW(hkey, NULL, NULL, RRF_RT_REG_SZ, &keytype, *association, &size);
+                    if(ret == ERROR_SUCCESS)
+                        hr = S_OK;
+                    else
+                    {
+                        CoTaskMemFree(*association);
+                        *association = NULL;
+                    }
+                }
+                else
+                    hr = E_OUTOFMEMORY;
+            }
+        }
+    }
+    else
+    {
+        ret = RegOpenKeyExW(HKEY_CURRENT_USER, assocations, 0, KEY_READ, &hkey);
+        if(ret == ERROR_SUCCESS)
+        {
+            if(type == AT_URLPROTOCOL)
+                lstrcpyW(path, urlassoc);
+            else if(type == AT_MIMETYPE)
+                lstrcpyW(path, mimeassoc);
+            else
+            {
+                WARN("Unsupported type (%d).\n", type);
+                RegCloseKey(hkey);
+                return hr;
+            }
+
+            lstrcatW(path, slash);
+            lstrcatW(path, query);
+            lstrcatW(path, slash);
+            lstrcatW(path, choice);
+
+            ret = RegGetValueW(hkey, path, propid, RRF_RT_REG_SZ, &keytype, NULL, &size);
+            if(ret == ERROR_SUCCESS)
+            {
+                *association = CoTaskMemAlloc(size);
+                if(*association)
+                {
+                    ret = RegGetValueW(hkey, path, propid, RRF_RT_REG_SZ, &keytype, *association, &size);
+                    if(ret == ERROR_SUCCESS)
+                        hr = S_OK;
+                    else
+                    {
+                        CoTaskMemFree(*association);
+                        *association = NULL;
+                    }
+                }
+                else
+                    hr = E_OUTOFMEMORY;
+            }
+        }
+    }
+
+    RegCloseKey(hkey);
+
+    return hr;
 }
 
 static HRESULT WINAPI ApplicationAssociationRegistration_QueryAppIsDefault(IApplicationAssociationRegistration* This, LPCWSTR query,
@@ -901,7 +1048,7 @@ HRESULT WINAPI QueryAssociations_Constructor(IUnknown *pUnkOuter, REFIID riid, L
     this->ref = 0;
     this->hkeySource = 0;
     this->hkeyProgID = 0;
-    if (FAILED(ret = IUnknown_QueryInterface((IUnknown *)this, riid, ppOutput))) SHFree( this );
+    if (FAILED(ret = IQueryAssociations_QueryInterface(&this->IQueryAssociations_iface, riid, ppOutput))) SHFree( this );
     TRACE("returning %p\n", *ppOutput);
     return ret;
 }
@@ -932,3 +1079,82 @@ HRESULT WINAPI ApplicationAssociationRegistration_Constructor(IUnknown *outer, R
     TRACE("returning 0x%x with %p\n", hr, *ppv);
     return hr;
 }
+
+static HRESULT WINAPI enumassochandlers_QueryInterface(IEnumAssocHandlers *iface, REFIID riid, void **obj)
+{
+    struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
+
+    TRACE("(%p %s %p)\n", This, debugstr_guid(riid), obj);
+
+    if (IsEqualIID(riid, &IID_IEnumAssocHandlers) ||
+        IsEqualIID(riid, &IID_IUnknown))
+    {
+        *obj = iface;
+        IEnumAssocHandlers_AddRef(iface);
+        return S_OK;
+    }
+
+    *obj = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI enumassochandlers_AddRef(IEnumAssocHandlers *iface)
+{
+    struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
+    ULONG ref = InterlockedIncrement(&This->ref);
+
+    TRACE("(%p)->(%u)\n", This, ref);
+    return ref;
+}
+
+static ULONG WINAPI enumassochandlers_Release(IEnumAssocHandlers *iface)
+{
+    struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
+    ULONG ref = InterlockedDecrement(&This->ref);
+
+    TRACE("(%p)->(%u)\n", This, ref);
+
+    if (!ref)
+        SHFree(This);
+
+    return ref;
+}
+
+static HRESULT WINAPI enumassochandlers_Next(IEnumAssocHandlers *iface, ULONG count, IAssocHandler **handlers,
+    ULONG *fetched)
+{
+    struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
+
+    FIXME("(%p)->(%u %p %p): stub\n", This, count, handlers, fetched);
+
+    return E_NOTIMPL;
+}
+
+static const IEnumAssocHandlersVtbl enumassochandlersvtbl = {
+    enumassochandlers_QueryInterface,
+    enumassochandlers_AddRef,
+    enumassochandlers_Release,
+    enumassochandlers_Next
+};
+
+/**************************************************************************
+ * SHAssocEnumHandlers            [SHELL32.@]
+ */
+HRESULT WINAPI SHAssocEnumHandlers(const WCHAR *extra, ASSOC_FILTER filter, IEnumAssocHandlers **enumhandlers)
+{
+    struct enumassochandlers *enumassoc;
+
+    FIXME("(%s %d %p): stub\n", debugstr_w(extra), filter, enumhandlers);
+
+    *enumhandlers = NULL;
+
+    enumassoc = SHAlloc(sizeof(*enumassoc));
+    if (!enumassoc)
+        return E_OUTOFMEMORY;
+
+    enumassoc->IEnumAssocHandlers_iface.lpVtbl = &enumassochandlersvtbl;
+    enumassoc->ref = 1;
+
+    *enumhandlers = &enumassoc->IEnumAssocHandlers_iface;
+    return S_OK;
+}
