@@ -81,7 +81,7 @@ void USER_Unlock(void)
  */
 void USER_CheckNotLock(void)
 {
-    if (user_section.OwningThread == ULongToHandle(GetCurrentThreadId()) && user_section.RecursionCount)
+    if (RtlIsCriticalSectionLockedByThread(&user_section))
     {
         ERR( "BUG: holding USER lock\n" );
         DebugBreak();
@@ -322,18 +322,24 @@ static void thread_detach(void)
  */
 BOOL WINAPI DllMain( HINSTANCE inst, DWORD reason, LPVOID reserved )
 {
+    static const WCHAR imm32_dllW[] = {'i','m','m','3','2','.','d','l','l',0};
+    static HMODULE imm32_module;
     BOOL ret = TRUE;
+
     switch(reason)
     {
     case DLL_PROCESS_ATTACH:
         user32_module = inst;
         ret = process_attach();
+        if(ret)
+            imm32_module = LoadLibraryW(imm32_dllW);
         break;
     case DLL_THREAD_DETACH:
         thread_detach();
         break;
     case DLL_PROCESS_DETACH:
         USER_unload_driver();
+        FreeLibrary(imm32_module);
         DeleteCriticalSection(&user_section);
         break;
     }
@@ -403,3 +409,23 @@ int WINAPI RegisterServicesProcess(DWORD ServicesProcessId)
     FIXME("(0x%x): stub\n", ServicesProcessId);
     return 0;
 }
+
+/***********************************************************************
+ *		ShutdownBlockReasonCreate (USER32.@)
+ */
+BOOL WINAPI ShutdownBlockReasonCreate(HWND hwnd, LPCWSTR reason)
+{
+    FIXME("(%p, %s): stub\n", hwnd, debugstr_w(reason));
+    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
+    return FALSE;
+}
+
+/***********************************************************************
+ *		ShutdownBlockReasonDestroy (USER32.@)
+ */
+BOOL WINAPI ShutdownBlockReasonDestroy(HWND hwnd)
+{
+    FIXME("(%p): stub\n", hwnd);
+    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
+    return FALSE;
+}
