@@ -52,6 +52,9 @@ UINT get_type_size( CIMTYPE type )
     {
     case CIM_BOOLEAN:
         return sizeof(int);
+    case CIM_SINT8:
+    case CIM_UINT8:
+        return sizeof(INT8);
     case CIM_SINT16:
     case CIM_UINT16:
         return sizeof(INT16);
@@ -111,6 +114,12 @@ HRESULT get_value( const struct table *table, UINT row, UINT column, LONGLONG *v
     case CIM_STRING:
         *val = (INT_PTR)*(const WCHAR **)ptr;
         break;
+    case CIM_SINT8:
+        *val = *(const INT8 *)ptr;
+        break;
+    case CIM_UINT8:
+        *val = *(const UINT8 *)ptr;
+        break;
     case CIM_SINT16:
         *val = *(const INT16 *)ptr;
         break;
@@ -215,6 +224,12 @@ HRESULT set_value( const struct table *table, UINT row, UINT column, LONGLONG va
     case CIM_STRING:
         *(WCHAR **)ptr = (WCHAR *)(INT_PTR)val;
         break;
+    case CIM_SINT8:
+        *(INT8 *)ptr = val;
+        break;
+    case CIM_UINT8:
+        *(UINT8 *)ptr = val;
+        break;
     case CIM_SINT16:
         *(INT16 *)ptr = val;
         break;
@@ -273,10 +288,15 @@ void free_row_values( const struct table *table, UINT row )
         if (!(table->columns[i].type & COL_FLAG_DYNAMIC)) continue;
 
         type = table->columns[i].type & COL_TYPE_MASK;
-        if (type == CIM_STRING || type == CIM_DATETIME || (type & CIM_FLAG_ARRAY))
+        if (type == CIM_STRING || type == CIM_DATETIME)
         {
             if (get_value( table, row, i, &val ) == S_OK) heap_free( (void *)(INT_PTR)val );
         }
+        else if (type & CIM_FLAG_ARRAY)
+        {
+            if (get_value( table, row, i, &val ) == S_OK)
+                destroy_array( (void *)(INT_PTR)val, type & CIM_TYPE_MASK );
+        }
     }
 }
 
