@@ -33,14 +33,6 @@
 
 DEFINE_GUID(IID_IXmlReaderInput, 0x0b3ccc9b, 0x9214, 0x428b, 0xa2, 0xae, 0xef, 0x3a, 0xa8, 0x71, 0xaf, 0xda);
 
-static HRESULT (WINAPI *pCreateXmlReader)(REFIID riid, void **ppvObject, IMalloc *pMalloc);
-static HRESULT (WINAPI *pCreateXmlReaderInputWithEncodingName)(IUnknown *stream,
-                                                        IMalloc *pMalloc,
-                                                        LPCWSTR encoding,
-                                                        BOOL hint,
-                                                        LPCWSTR base_uri,
-                                                        IXmlReaderInput **ppInput);
-
 static WCHAR *a2w(const char *str)
 {
     int len = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0);
@@ -96,15 +88,9 @@ static void ok_pos_(IXmlReader *reader, int line, int pos, int line_broken,
         broken_state = broken((line_broken == -1 ? line : line_broken) == l &&
                               (pos_broken == -1 ? pos : pos_broken) == p);
 
-    if (todo)
-        todo_wine
+    todo_wine_if (todo)
         ok_(__FILE__, _line_)((l == line && pos == p) || broken_state,
                             "Expected (%d,%d), got (%d,%d)\n", line, pos, l, p);
-    else
-    {
-        ok_(__FILE__, _line_)((l == line && pos == p) || broken_state,
-                            "Expected (%d,%d), got (%d,%d)\n", line, pos, l, p);
-    }
 }
 #define ok_pos(reader, l, p, l_brk, p_brk, todo) ok_pos_(reader, l, p, l_brk, p_brk, todo, __LINE__)
 
@@ -147,12 +133,8 @@ static void ok_iids_(const input_iids_t *iids, const IID **expected, const IID *
 
     while (expected[i++]) size++;
 
-    if (todo) {
-        todo_wine
-            ok_(__FILE__, line)(iids->count == size, "Sequence size mismatch (%d), got (%d)\n", size, iids->count);
-    }
-    else
-       ok_(__FILE__, line)(iids->count == size, "Sequence size mismatch (%d), got (%d)\n", size, iids->count);
+    todo_wine_if (todo)
+        ok_(__FILE__, line)(iids->count == size, "Sequence size mismatch (%d), got (%d)\n", size, iids->count);
 
     if (iids->count != size) return;
 
@@ -247,13 +229,7 @@ static void test_read_state_(IXmlReader *reader, XmlReadState expected,
     else
         broken_state = broken(exp_broken == state);
 
-    if (todo)
-    {
-    todo_wine
-        ok_(__FILE__, line)(state == expected || broken_state, "Expected (%s), got (%s)\n",
-                                   state_to_str(expected), state_to_str(state));
-    }
-    else
+    todo_wine_if (todo)
         ok_(__FILE__, line)(state == expected || broken_state, "Expected (%s), got (%s)\n",
                                    state_to_str(expected), state_to_str(state));
 }
@@ -383,27 +359,51 @@ static const ISequentialStreamVtbl teststreamvtbl =
     teststream_Write
 };
 
-static BOOL init_pointers(void)
+static HRESULT WINAPI resolver_QI(IXmlResolver *iface, REFIID riid, void **obj)
 {
-    /* don't free module here, it's to be unloaded on exit */
-    HMODULE mod = LoadLibraryA("xmllite.dll");
+    ok(0, "unexpected call, riid %s\n", wine_dbgstr_guid(riid));
 
-    if (!mod)
+    if (IsEqualIID(riid, &IID_IXmlResolver) || IsEqualIID(riid, &IID_IUnknown))
     {
-        win_skip("xmllite library not available\n");
-        return FALSE;
+        *obj = iface;
+        IXmlResolver_AddRef(iface);
+        return S_OK;
     }
 
-#define MAKEFUNC(f) if (!(p##f = (void*)GetProcAddress(mod, #f))) return FALSE;
-    MAKEFUNC(CreateXmlReader);
-    MAKEFUNC(CreateXmlReaderInputWithEncodingName);
-#undef MAKEFUNC
-
-    return TRUE;
+    *obj = NULL;
+    return E_NOINTERFACE;
 }
 
+static ULONG WINAPI resolver_AddRef(IXmlResolver *iface)
+{
+    return 2;
+}
+
+static ULONG WINAPI resolver_Release(IXmlResolver *iface)
+{
+    return 1;
+}
+
+static HRESULT WINAPI resolver_ResolveUri(IXmlResolver *iface, const WCHAR *base_uri,
+    const WCHAR *public_id, const WCHAR *system_id, IUnknown **input)
+{
+    ok(0, "unexpected call\n");
+    return E_NOTIMPL;
+}
+
+static const IXmlResolverVtbl resolvervtbl =
+{
+    resolver_QI,
+    resolver_AddRef,
+    resolver_Release,
+    resolver_ResolveUri
+};
+
+static IXmlResolver testresolver = { &resolvervtbl };
+
 static void test_reader_create(void)
 {
+    IXmlResolver *resolver;
     HRESULT hr;
     IXmlReader *reader;
     IUnknown *input;
@@ -413,11 +413,11 @@ static void test_reader_create(void)
     /* crashes native */
     if (0)
     {
-        pCreateXmlReader(&IID_IXmlReader, NULL, NULL);
-        pCreateXmlReader(NULL, (void**)&reader, NULL);
+        CreateXmlReader(&IID_IXmlReader, NULL, NULL);
+        CreateXmlReader(NULL, (void**)&reader, NULL);
     }
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     test_read_state(reader, XmlReadState_Closed, -1, FALSE);
@@ -427,6 +427,26 @@ static void test_reader_create(void)
     ok(hr == S_FALSE, "got %08x\n", hr);
     ok(nodetype == XmlNodeType_None, "got %d\n", nodetype);
 
+    resolver = (void*)0xdeadbeef;
+    hr = IXmlReader_GetProperty(reader, XmlReaderProperty_XmlResolver, (LONG_PTR*)&resolver);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(resolver == NULL, "got %p\n", resolver);
+
+    hr = IXmlReader_SetProperty(reader, XmlReaderProperty_XmlResolver, 0);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IXmlReader_SetProperty(reader, XmlReaderProperty_XmlResolver, (LONG_PTR)&testresolver);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    resolver = NULL;
+    hr = IXmlReader_GetProperty(reader, XmlReaderProperty_XmlResolver, (LONG_PTR*)&resolver);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(resolver == &testresolver, "got %p\n", resolver);
+    IXmlResolver_Release(resolver);
+
+    hr = IXmlReader_SetProperty(reader, XmlReaderProperty_XmlResolver, 0);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
     dtd = 2;
     hr = IXmlReader_GetProperty(reader, XmlReaderProperty_DtdProcessing, (LONG_PTR*)&dtd);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
@@ -470,9 +490,9 @@ static void test_readerinput(void)
     HRESULT hr;
     LONG ref;
 
-    hr = pCreateXmlReaderInputWithEncodingName(NULL, NULL, NULL, FALSE, NULL, NULL);
+    hr = CreateXmlReaderInputWithEncodingName(NULL, NULL, NULL, FALSE, NULL, NULL);
     ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
-    hr = pCreateXmlReaderInputWithEncodingName(NULL, NULL, NULL, FALSE, NULL, &reader_input);
+    hr = CreateXmlReaderInputWithEncodingName(NULL, NULL, NULL, FALSE, NULL, &reader_input);
     ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
 
     hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
@@ -481,7 +501,7 @@ static void test_readerinput(void)
     ref = IStream_AddRef(stream);
     ok(ref == 2, "Expected 2, got %d\n", ref);
     IStream_Release(stream);
-    hr = pCreateXmlReaderInputWithEncodingName((IUnknown*)stream, NULL, NULL, FALSE, NULL, &reader_input);
+    hr = CreateXmlReaderInputWithEncodingName((IUnknown*)stream, NULL, NULL, FALSE, NULL, &reader_input);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     hr = IUnknown_QueryInterface(reader_input, &IID_IStream, (void**)&stream2);
@@ -496,7 +516,7 @@ static void test_readerinput(void)
     IStream_Release(stream);
 
     /* try ::SetInput() with valid IXmlReaderInput */
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     ref = IUnknown_AddRef(reader_input);
@@ -562,7 +582,7 @@ static void test_readerinput(void)
     ref = IUnknown_AddRef(input);
     ok(ref == 2, "Expected 2, got %d\n", ref);
     IUnknown_Release(input);
-    hr = pCreateXmlReaderInputWithEncodingName(input, NULL, NULL, FALSE, NULL, &reader_input);
+    hr = CreateXmlReaderInputWithEncodingName(input, NULL, NULL, FALSE, NULL, &reader_input);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
     ok_iids(&input_iids, empty_seq, NULL, FALSE);
     /* IXmlReaderInput stores stream interface as IUnknown */
@@ -570,7 +590,7 @@ static void test_readerinput(void)
     ok(ref == 3, "Expected 3, got %d\n", ref);
     IUnknown_Release(input);
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     input_iids.count = 0;
@@ -601,7 +621,7 @@ static void test_readerinput(void)
     ok_iids(&input_iids, setinput_readerinput, NULL, FALSE);
 
     /* another reader */
-    hr = pCreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader2, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader2, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     /* resolving from IXmlReaderInput to IStream/ISequentialStream is done at
@@ -624,7 +644,7 @@ static void test_reader_state(void)
     XmlNodeType nodetype;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     /* invalid arguments */
@@ -664,7 +684,7 @@ static void test_read_xmldeclaration(void)
     BOOL ret;
     const WCHAR *val;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     /* position methods with Null args */
@@ -885,7 +905,7 @@ static void test_read_comment(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -967,7 +987,7 @@ static void test_read_pi(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1083,7 +1103,7 @@ static void test_read_full(void)
     HRESULT hr;
     int i;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     stream = create_stream_on_data(test->xml, strlen(test->xml)+1);
@@ -1132,7 +1152,7 @@ static void test_read_dtd(void)
     UINT len, count;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     hr = IXmlReader_SetProperty(reader, XmlReaderProperty_DtdProcessing, DtdProcessing_Parse);
@@ -1225,7 +1245,7 @@ static void test_read_element(void)
     UINT depth;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1356,7 +1376,7 @@ static void test_read_pending(void)
     HRESULT hr;
     int c;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got 0x%08x\n", hr);
 
     hr = IXmlReader_SetInput(reader, (IUnknown*)&teststream);
@@ -1394,7 +1414,7 @@ static void test_readvaluechunk(void)
     HRESULT hr;
     UINT c;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     stream = create_stream_on_data(testA, sizeof(testA));
@@ -1449,7 +1469,7 @@ static void test_read_cdata(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1515,14 +1535,7 @@ static void test_read_cdata(void)
             ok(hr == S_OK, "got 0x%08x\n", hr);
 
             str_exp = a2w(test->value);
-            if (test->todo)
-            {
-            todo_wine {
-                ok(len == strlen(test->value), "got %u\n", len);
-                ok(!lstrcmpW(str, str_exp), "got %s\n", wine_dbgstr_w(str));
-            }
-            }
-            else
+            todo_wine_if (test->todo)
             {
                 ok(len == strlen(test->value), "got %u\n", len);
                 ok(!lstrcmpW(str, str_exp), "got %s\n", wine_dbgstr_w(str));
@@ -1549,7 +1562,7 @@ static void test_read_text(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1615,14 +1628,7 @@ static void test_read_text(void)
             ok(hr == S_OK, "got 0x%08x\n", hr);
 
             str_exp = a2w(test->value);
-            if (test->todo)
-            {
-            todo_wine {
-                ok(len == strlen(test->value), "got %u\n", len);
-                ok(!lstrcmpW(str, str_exp), "got %s\n", wine_dbgstr_w(str));
-            }
-            }
-            else
+            todo_wine_if (test->todo)
             {
                 ok(len == strlen(test->value), "got %u\n", len);
                 ok(!lstrcmpW(str, str_exp), "got %s\n", wine_dbgstr_w(str));
@@ -1654,7 +1660,7 @@ static void test_isemptyelement(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1712,7 +1718,7 @@ static void test_read_attribute(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1781,9 +1787,6 @@ static void test_read_attribute(void)
 
 START_TEST(reader)
 {
-    if (!init_pointers())
-       return;
-
     test_reader_create();
     test_readerinput();
     test_reader_state();
