@@ -30,6 +30,43 @@ extern "C" {
 #define EVENT_TRACE_CONTROL_UPDATE    2
 #define EVENT_TRACE_CONTROL_FLUSH     3
 
+#define TRACE_LEVEL_NONE              0
+#define TRACE_LEVEL_CRITICAL          1
+#define TRACE_LEVEL_FATAL             1
+#define TRACE_LEVEL_ERROR             2
+#define TRACE_LEVEL_WARNING           3
+#define TRACE_LEVEL_INFORMATION       4
+#define TRACE_LEVEL_VERBOSE           5
+
+#define EVENT_TRACE_FILE_MODE_NONE             0x00000000
+#define EVENT_TRACE_FILE_MODE_SEQUENTIAL       0x00000001
+#define EVENT_TRACE_FILE_MODE_CIRCULAR         0x00000002
+#define EVENT_TRACE_FILE_MODE_APPEND           0x00000004
+#define EVENT_TRACE_FILE_MODE_NEWFILE          0x00000008
+#define EVENT_TRACE_FILE_MODE_PREALLOCATE      0x00000020
+#define EVENT_TRACE_NONSTOPPABLE_MODE          0x00000040
+#define EVENT_TRACE_SECURE_MODE                0x00000080
+#define EVENT_TRACE_REAL_TIME_MODE             0x00000100
+#define EVENT_TRACE_DELAY_OPEN_FILE_MODE       0x00000200
+#define EVENT_TRACE_BUFFERING_MODE             0x00000400
+#define EVENT_TRACE_PRIVATE_LOGGER_MODE        0x00000800
+#define EVENT_TRACE_ADD_HEADER_MODE            0x00001000
+#define EVENT_TRACE_USE_KBYTES_FOR_SIZE        0x00002000
+#define EVENT_TRACE_USE_GLOBAL_SEQUENCE        0x00004000
+#define EVENT_TRACE_USE_LOCAL_SEQUENCE         0x00008000
+#define EVENT_TRACE_RELOG_MODE                 0x00010000
+#define EVENT_TRACE_PRIVATE_IN_PROC            0x00020000
+#define EVENT_TRACE_MODE_RESERVED              0x00100000
+#define EVENT_TRACE_STOP_ON_HYBRID_SHUTDOWN    0x00400000
+#define EVENT_TRACE_PERSIST_ON_HYBRID_SHUTDOWN 0x00800000
+#define EVENT_TRACE_USE_PAGED_MEMORY           0x01000000
+#define EVENT_TRACE_SYSTEM_LOGGER_MODE         0x02000000
+#define EVENT_TRACE_INDEPENDENT_SESSION_MODE   0x08000000
+#define EVENT_TRACE_NO_PER_PROCESSOR_BUFFERING 0x10000000
+#define EVENT_TRACE_ADDTO_TRIAGE_DUMP          0x80000000
+
+DEFINE_GUID (SystemTraceControlGuid, 0x9e814aad, 0x3204, 0x11d2, 0x9a, 0x82, 0x00, 0x60, 0x08, 0xa8, 0x69, 0x39);
+
 typedef ULONG64 TRACEHANDLE, *PTRACEHANDLE;
 
 struct _EVENT_TRACE_LOGFILEA;
@@ -207,7 +244,7 @@ typedef struct _EVENT_TRACE_PROPERTIES
     ULONG LogBuffersLost;
     ULONG RealTimeBuffersLost;
     HANDLE LoggerThreadId;
-    ULONG LoggerFileNameOffset;
+    ULONG LogFileNameOffset;
     ULONG LoggerNameOffset;
 } EVENT_TRACE_PROPERTIES, *PEVENT_TRACE_PROPERTIES;
 
@@ -221,6 +258,9 @@ ULONG WINAPI EnableTrace(ULONG,ULONG,ULONG,LPCGUID,TRACEHANDLE);
 ULONG WINAPI FlushTraceA(TRACEHANDLE,LPCSTR,PEVENT_TRACE_PROPERTIES);
 ULONG WINAPI FlushTraceW(TRACEHANDLE,LPCWSTR,PEVENT_TRACE_PROPERTIES);
 #define      FlushTrace WINELIB_NAME_AW(FlushTrace)
+ULONG WINAPI GetTraceEnableFlags(TRACEHANDLE);
+UCHAR WINAPI GetTraceEnableLevel(TRACEHANDLE);
+TRACEHANDLE WINAPI GetTraceLoggerHandle(PVOID);
 ULONG WINAPI QueryAllTracesA(PEVENT_TRACE_PROPERTIES*,ULONG,PULONG);
 ULONG WINAPI QueryAllTracesW(PEVENT_TRACE_PROPERTIES*,ULONG,PULONG);
 #define      QueryAllTraces WINELIB_NAME_AW(QueryAllTraces)
