@@ -45,7 +45,17 @@ static void add_cmdline_defines(void)
 
     for (def = cmdline_defines; def; def = def->next)
     {
-        if (def->value) pp_add_define( pp_xstrdup(def->name), pp_xstrdup(def->value) );
+        if (def->value) pp_add_define( def->name, def->value );
+    }
+}
+
+static void del_cmdline_defines(void)
+{
+    struct define *def;
+
+    for (def = cmdline_defines; def; def = def->next)
+    {
+        if (def->value) pp_del_define( def->name );
     }
 }
 
@@ -56,20 +66,28 @@ static void add_special_defines(void)
     char buf[32];
 
     strftime(buf, sizeof(buf), "\"%b %d %Y\"", localtime(&now));
-    pp_add_define( pp_xstrdup("__DATE__"), pp_xstrdup(buf) );
+    pp_add_define( "__DATE__", buf );
 
     strftime(buf, sizeof(buf), "\"%H:%M:%S\"", localtime(&now));
-    pp_add_define( pp_xstrdup("__TIME__"), pp_xstrdup(buf) );
+    pp_add_define( "__TIME__", buf );
 
-    ppp = pp_add_define( pp_xstrdup("__FILE__"), pp_xstrdup("") );
+    ppp = pp_add_define( "__FILE__", "" );
     if(ppp)
         ppp->type = def_special;
 
-    ppp = pp_add_define( pp_xstrdup("__LINE__"), pp_xstrdup("") );
+    ppp = pp_add_define( "__LINE__", "" );
     if(ppp)
         ppp->type = def_special;
 }
 
+static void del_special_defines(void)
+{
+    pp_del_define( "__DATE__" );
+    pp_del_define( "__TIME__" );
+    pp_del_define( "__FILE__" );
+    pp_del_define( "__LINE__" );
+}
+
 
 /* add a define to the preprocessor list */
 int wpp_add_define( const char *name, const char *value )
@@ -168,8 +186,8 @@ int wpp_parse( const char *input, FILE *output )
     int ret;
 
     pp_status.input = NULL;
-    pp_status.line_number = 0;
-    pp_status.char_number = 0;
+    pp_status.line_number = 1;
+    pp_status.char_number = 1;
     pp_status.state = 0;
 
     ret = pp_push_define_state();
@@ -182,11 +200,13 @@ int wpp_parse( const char *input, FILE *output )
     else if (!(pp_status.file = wpp_callbacks->open(input, 1)))
     {
         ppy_error("Could not open %s\n", input);
+        del_special_defines();
+        del_cmdline_defines();
         pp_pop_define_state();
         return 2;
     }
 
-    pp_status.input = input;
+    pp_status.input = input ? pp_xstrdup(input) : NULL;
 
     ppy_out = output;
     pp_writestring("# 1 \"%s\" 1\n", input ? input : "");
@@ -195,51 +215,20 @@ int wpp_parse( const char *input, FILE *output )
     /* If there were errors during processing, return an error code */
     if (!ret && pp_status.state) ret = pp_status.state;
 
-    if (input) wpp_callbacks->close(pp_status.file);
+    if (input)
+    {
+	wpp_callbacks->close(pp_status.file);
+	free(pp_status.input);
+    }
     /* Clean if_stack, it could remain dirty on errors */
     while (pp_get_if_depth()) pp_pop_if();
+    del_special_defines();
+    del_cmdline_defines();
     pp_pop_define_state();
     return ret;
 }
 
 
-/* parse into a temporary file */
-int wpp_parse_temp( const char *input, const char *output_base, char **output_name )
-{
-    FILE *output;
-    int ret, fd;
-    char *temp_name;
-
-    if (!output_base || !output_base[0]) output_base = "wpptmp";
-
-    temp_name = pp_xmalloc( strlen(output_base) + 8 );
-    if(!temp_name)
-        return 1;
-    strcpy( temp_name, output_base );
-    strcat( temp_name, ".XXXXXX" );
-
-    if((fd = mkstemps( temp_name, 0 )) == -1)
-    {
-        ppy_error("Could not generate a temp name from %s\n", temp_name);
-        free( temp_name );
-        return 2;
-    }
-
-    if (!(output = fdopen(fd, "wt")))
-    {
-        ppy_error("Could not open fd %s for writing\n", temp_name);
-        close( fd );
-        unlink( temp_name );
-        free( temp_name );
-        return 2;
-    }
-
-    *output_name = temp_name;
-    ret = wpp_parse( input, output );
-    fclose( output );
-    return ret;
-}
-
 void wpp_set_callbacks( const struct wpp_callbacks *callbacks )
 {
     wpp_callbacks = callbacks;
