@@ -20,6 +20,7 @@
 
 #include <ctype.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <windows.h>
 #include "regproc.h"
 
@@ -141,29 +142,29 @@ static BOOL PerformRegAction(REGEDIT_ACTION action, LPSTR s)
 
                 if (strcmp(filename, "-") == 0)
                 {
-                    reg_file=stdin;
+                    reg_file = stdin;
                 }
                 else
                 {
                     int size;
 
-                    size=SearchPath(NULL, filename, NULL,0, NULL, NULL);
-                    if (size>0)
+                    size = SearchPathA(NULL, filename, NULL, 0, NULL, NULL);
+                    if (size > 0)
                     {
-                        realname=HeapAlloc(GetProcessHeap(), 0, size);
-                        size=SearchPath(NULL, filename, NULL, size, realname, NULL);
+                        realname = HeapAlloc(GetProcessHeap(), 0, size);
+                        size = SearchPathA(NULL, filename, NULL, size, realname, NULL);
                     }
-                    if (size==0)
+                    if (size == 0)
                     {
                         fprintf(stderr, "%s: File not found \"%s\" (%d)\n",
                                 getAppName(), filename, GetLastError());
                         exit(1);
                     }
-                    reg_file = fopen(realname, "r");
-                    if (reg_file==NULL)
+                    reg_file = fopen(realname, "rb");
+                    if (reg_file == NULL)
                     {
                         perror("");
-                        fprintf(stderr,"%s: Can't open file \"%s\"\n", getAppName(), filename);
+                        fprintf(stderr, "%s: Can't open file \"%s\"\n", getAppName(), filename);
                         exit(1);
                     }
                 }
@@ -196,6 +197,7 @@ static BOOL PerformRegAction(REGEDIT_ACTION action, LPSTR s)
         }
     case ACTION_EXPORT: {
             CHAR filename[MAX_PATH];
+            WCHAR* filenameW;
 
             filename[0] = '\0';
             get_file_name(&s, filename);
@@ -205,14 +207,19 @@ static BOOL PerformRegAction(REGEDIT_ACTION action, LPSTR s)
                 exit(1);
             }
 
+            filenameW = GetWideString(filename);
             if (s[0]) {
                 CHAR reg_key_name[KEY_MAX_LEN];
+                WCHAR* reg_key_nameW;
 
                 get_file_name(&s, reg_key_name);
-                export_registry_key(filename, reg_key_name);
+                reg_key_nameW = GetWideString(reg_key_name);
+                export_registry_key(filenameW, reg_key_nameW, REG_FORMAT_4);
+                HeapFree(GetProcessHeap(), 0, reg_key_nameW);
             } else {
-                export_registry_key(filename, NULL);
+                export_registry_key(filenameW, NULL, REG_FORMAT_4);
             }
+            HeapFree(GetProcessHeap(), 0, filenameW);
             break;
         }
     default:
